C.......PROGRAM COSIM                                                   
C                                                                       
C       PROGRAM COSIM :      CO-CONDITIONAL SIMULATION OF SPATIAL       
C                            COREGIONALIZATION.                         
C                                                                       
C       PROGRAM AUTHOR:      DR. JAMES RUSSELL CARR                     
C                            DEPARTMENT OF GEOLOGICAL ENGINEERING       
C                            UNIVERSITY OF MISSOURI                     
C                            ROLLA, MISSOURI, U.S.A.  65401             
C                            TELEPHONE:  (314) 341-4867                 
C                                                                       
C       PROGRAM LANGUAGE:    FORTRAN IV (IBM 4331)                      
C               STORAGE :    130 K                                      
C               VERSION :    JUNE, 1984                                 
C                                                                       
C       GENERAL INFORMATION:                                            
C                                                                       
C                            PROGRAM COSIM PROVIDES THE FOLLOWING       
C                            CAPABILITIES:                              
C                                                                       
C                            1.  NON-CONDITIONAL SIMULATION OF N        
C                                VARIABLES (N = 1 TO 5).                
C                            2.  A PRINT OF NON-CONDITIONAL RESULTS     
C                            3.  VARIOGRAM OF NON-CONDITIONAL RESULTS   
C                            4.  HISTOGRAMS OF RANDOM NUMBERS AND OF    
C                                NON-CONDITIONAL SIMULATIONS            
C                            5.  TRANSFORMATION OF NON-                 
C                                CONDITIONAL SIMULATIONS                
C                            6.  A PRINT OF TRANSFORMATION RESULTS      
C                                ALONG WITH HISTOGRAMS AND VARIOGRAMS   
C                            7.  CONDITIONING THROUGH CO-KRIGING        
C                            8.  HISTOGRAMS AND VARIOGRAMS OF           
C                                CONDITIONING RESULTS                   
C                            9.  A PRINT OF CONDITIONING RESULTS        
C                           10.  COMPUTATION OF CROSS-VARIOGRAMS OF     
C                                COREGIONALIZATIONS.                    
C                           11.  A PRINT OF SIMULATED VALUES AT TEST    
C                                LOCATIONS ( 20 LOCATIONS MAX. )        
C                                                                       
C                                                                       
C       PROGRAM LIMITATIONS:                                            
C                                                                       
C                            1.  SPHERICAL SPATIAL LAW ONLY             
C                            2.  LINEAR OR HERMITIAN TRANSFORMATION     
C                            3.  EAST-WEST VARIOGRAM ONLY               
C                            4.  10 BIN HISTOGRAM COMPUTATION           
C                            5.  65 X 65 SIMULATION, MAXIMUM            
C                            6.  16 BIT MINIMUM MACHINE CAPABILITY      
C                                SUPPORTING DOUBLE PRECISION USAGE      
C                            7.  10-STEP DIVISION PRINT CODE            
C                                FOR PRINTER PICTURE OF SIMULATION      
C                                RESULTS                                
C                            8.  FOR CONDITIONING, THE FOLLOWING        
C                                LIST GIVES MAXIMUM ALLOWANCES FOR      
C                                THE NUMBER OF CONDITIONING DATA:       
C                                                                       
C                                NO. OF VARIABLES      NO. OF DATA      
C                                                                       
C                                         1                 99          
C                                         2                 49          
C                                         3                 32          
C                                         4                 24          
C                                         5                 19          
C                                                                       
C                            9.  A MAXIMUM OF 5 VARIABLES CAN BE        
C                                SIMULATED UNDER CURRENT DIMENSIONING   
C                           10.  A MAXIMUM OF 20 TEST LOCATIONS CAN     
C                                BE SPECIFIED                           
C                                                                       
C   *****************************************************************   
C                      GUIDE TO DATA INPUT                              
C   *****************************************************************   
C                                                                       
C       DATA ACQUISITION IS CARRIED OUT IN SUBROUTINE INIT              
C                                                                       
CGJR  Data added by Greg Ruskauff.
C       Record 0.  (Free Format).  Number of realizations (<32,000)
C       Record 0a. (Free Format).  Save flags.
C                   READ(5,*) SAVEUN, SAVECON (T or F)
C       Record 0b. (Free Format).  Output files.
C                   READ(5,*) UNCONSAVE, CONSAVE
C                 You MUST enter a character string (up to 50 chars)
C                 for each name, even if not used.  No quotes are necessary.
C       Record 0c.  (Free Format).  Back transform flag.
C                   READ(5,*) LOGBACK (T or F)
C                  If True, do base 10 back transform.
CGJR End of added data.
C
C       RECORD 1.  (FREE FORMAT).  NUMBER OF VARIABLES.                 
C                                                                       
C                  READ (5,*) MVAR                                      
C                             MVAR = NO. OF VARIABLES  (1-5)            
C                             IF MVAR = 1, COSIM IS A STANDARD          
C                             CONDITIONAL SIMULATION PROGRAM            
C                                                                       
C       RECORD 2.  (FREE FORMAT).  GRID DIMENSIONS                      
C                                                                       
C                  READ (5,*) NROW,NCOL,XDIM,YDIM                       
C                             NROW = NO. OF ROWS IN SIMULATION MODEL    
C                             NCOL = NO. OF COLS IN SIMULATION MODEL    
C                             XDIM = INCREMENT IN X BETWEEN COLUMNS     
C                             YDIM = INCREMENT IN Y BETWEEN ROWS        
C                                                                       
C       RECORD 3.  (FREE FORMAT).  GEOGRAPHIC REGISTRATION.             
C                                                                       
C                  READ (5,*) YMAX,XMIN                                 
C                             YMAX = Y-COORDINATE OF UPPER LEFT CORNER  
C                                    OF SIMULATION MODEL                
C                             XMIN = X-COORDINATE OF THIS CORNER        
C                                                                       
C       RECORD 4.  (FREE FORMAT).  BEGINNING RANDOM NUMBER SEED.        
C                                                                       
C                  READ(5,*)  RSEED                                     
C                             RSEED = BEGINNING RANDOM NUMBER SEED.     
C                             ANY ODD NUMBER IN THE RANGE:              
C                             1 TO 2147483647                           
C                                                                       
C       RECORD(S) 5.  (FREE FORMAT).  MODEL TRANSFORMATION AND ETC.     
C                                                                       
C                  READ(5,*) ITRANS                                     
C                                   ITRANS = 0 --- LINEAR TRANS.        
C                                          = 1 --- HERMITIAN TRANS.     
C                  IF ITRANS = 0:                                       
C                  DO 10 I = 1,MVAR                                     
C                  READ (5,*) LVAR(I),LMEAN(I),RANG(I),CNUG(I)          
C               10 CONTINUE                                             
C                             LVAR(I) = DESIRED VARIANCE OF VARIABLE I  
C                             LMEAN(I)= DESIRED MEAN OF VARIABLE I      
C                             RANG(I) = DESIRED RANGE OF SPATIAL LAW    
C                             CNUG(I) = DESIRED NUGGET EFFECT           
C                                                                       
C                  IF ITRANS = 1:                                       
C                  DO 15 I = 1,MVAR                                     
C                  READ(5,*) (COEF(JK,I),JK = 1,10),RANG(I)             
C               15 CONTINUE                                             
C                            COEF = 10 COEF. OF HERMITIAN TRANS.        
C                            RANG = RANGE OF SPATIAL STRUCTURE          
C                                                                       
C       RECORD 6.  (FREE FORMAT). CONDITIONING OPTION.                  
C                                                                       
C                  READ (5,*) NSTOP                                     
C                             NSTOP =  0)  NON-CONDITIONAL SIMULATION   
C                                      1)  CONDITIONAL SIMULATION       
C                                                                       
C       RECORD(S) 7. (FREE FORMAT). REQUIRED ONLY IF NSTOP = 1.         
C                  VARIOGRAM PARAMETERS.                                
C                                                                       
C                  DO 10 I = 1,MVAR                                     
C                  READ(5,*) CO(I),C(I),RANGE(I),ANIS(I),RATIO(I)       
C               10 CONTINUE                                             
C                           CO(I)  = NUGGET OF VARIOGRAM FOR VAR. I     
C                            C(I)  = SILL OF VARIOGRAM FOR VARIABLE I   
C                        RANGE(I)  = RANGE OF VARIOGRAM FOR VARIABLE I  
C                         ANIS(I)  = ANGLE OF SPATIAL ANISOTROPY        
C                        RATIO(I)  = RANGE,LONG / RANGE, SHORT          
C                                    ISOTROPY: ANIS = 0; RATIO = 1.     
C                                                                       
C       RECORD(S) 8. (FREE FORMAT). REQUIRED IF MVAR .GT. 1 AND         
C                                   NSTOP = 1.                          
C                  CROSS - VARIOGRAM PARAMETERS.                        
C                                                                       
C                  NN = MVAR * (MVAR - 1)) / 2                          
C                  DO 10 I = 1,NN                                       
C                  READ(5,*) CCO(I),CC(I),CRANGE(I),CANIS(I),CRATIO(I)  
C               10 CONTINUE                                             
C                           CCO(I) = NUGGET OF CROSS-VARIOGRAM, I       
C                            CC(I) = SILL OF CROSS-VARIOGRAM, I         
C                        CRANGE(I) = RANGE OF CROSS-VARIOGRAM,I         
C                         CANIS(I) = ANGLE OF ANISOTROPY                
C                        CRATIO(I) = RANGE, LONG / RANGE, SHORT         
C                                    ISOTROPY: CANIS=0; CRATIO=1.       
C                                                                       
C       RECORD 9. (FREE FORMAT).  REQUIRED ONLY IF NSTOP = 1.           
C                                                                       
C                  READ(5,*) IKRIG                                      
C                            IKRIG = 0) ALL VARIABLES FULLY SAMPLED     
C                            IKRIG = 1) SOME VARIABLES UNDERSAMPLED     
C                                                                       
C       RECORD(S) 10. (FREE FORMAT).  REQUIRED ONLY IF NSTOP = 1.       
C                  CONDITIONING DATA                                    
C                                                                       
C                  (  GO TO LOOP  )                                     
C                  READ (5,*) D1,D2,(DAT(JK), JK = 1,MVAR)              
C                             D1 = Y-COORDINATE OF CONDITIONING PT.     
C                             D2 = X-COORDINATE OF CONDITIONING PT.     
C                             THEN, 1 TO MVAR DATA VALUES               
C                  LAST RECORD: 0.0,0.0,MVAR*0.0                        
C
C       Note that Y NOT X is the first coordinate for the conditioning data. GJR.
C
C                                                                       
C       RECORD 11. (FREE FORMAT).  REQUIRED ONLY IF NSTOP = 1.          
C                                                                       
C                  READ (5,*) KTEST                                     
C                             KTEST = 0) NO TESTING                     
C                                     1) TESTING                        
C                                                                       
C       RECORD(S) 12. (FREE FORMAT).  REQUIRED IF NSTOP AND KTEST = 1.  
C                  TEST LOCATIONS                                       
C                                                                       
C                  ( GO TO LOOP)                                        
C                  READ (5,*) TX,TY                                     
C                             (TX,TY)=COORDINATES OF TEST LOCATION      
C                              TX=TY=0.0 STOPS ACQUISITION              
C                                                                       
C   ******************************************************************  
C                  END OF DATA INPUT GUIDE                              
C   ******************************************************************  
C                                                                       
C                                                                       
C                                                                       

C------------------------------------------------------------------------
C       This copy of COSIM was ported to the IBM PC and Macintosh 
C       environments by Greg Ruskauff.  If there are any errors contact me at:
C
C               INTERA Inc.
C               1650 University Blvd. NE, Ste. 300
C               Albuquerque, NM  87102-1732  USA
C               (505) 246-1600
C               INTERANM@NMIA.COM
C               HTTP://THALES.NMIA.COM/~INTERAGS/HOME/HOME.HTML
C
C       The Microsoft FORTRAN compiler version 5.0 for DOS, Lahey FORTRAN v. 5.0,
C       and Language Systems
C       MPW FORTRAN for the Macintosh were used to test the ported code.
C
C       If your going to use this on a Mac under Language Systems the 
C       WRITE(*,*)s need to write to unit 5 instead of *.
C       
C       I have left comments where I made modifications.
C
C       Dr. Carr has moved and can now be reached at:
C
C       University of Nevada
C       Dept. of Geological Sciences/168
C       Geological Engineering Division
C       Reno, Nevada   89557-0138
C       (702) 784-1766
C


C Explicit PROGRAM statement added by GJR.
        PROGRAM COSIM
        
        INCLUDE 'SIZES.INC'

CGJR Old COMMONs were hardcoded, I added an INCLUDE file that contains PARAMETERs
CGJR for all array dimensions.

C        COMMON /SIM/ RF(65,65,6)                                        
         COMMON /SIM/ RF(W_RFX,W_RFY,W_RFZ)                                        
        
C        COMMON /DVEC/ ICOS(3,15)         
         COMMON /DVEC/ ICOS(W_ICOS1,W_ICOS2)         
                               
C        COMMON /DAT2/ X(100),Y(100),DAT(100,5)                          
         COMMON /DAT2/ X(W_X),Y(W_Y),DAT(W_DAT1,W_DAT2)                          

C        COMMON /FORM/ A(100,100)                                        
         COMMON /FORM/ A(W_AX,W_AY)                                        

C        COMMON /VAR/  CO(5),C(5),RANGE(5),ANIS(5),RATIO(5)              
         COMMON /VAR/ CO(W_CO),C(W_C),RANGE(W_RNG),
     1                  ANIS(W_AN),RATIO(W_RAT)

C        COMMON /CVAR/ CCO(10),CC(10),CRANGE(10),CANIS(10),CRATIO(10)    
         COMMON /CVAR/ CCO(W_CCO),CC(W_CC),CRANGE(W_CRAN),
     1                  CANIS(W_CAN),CRATIO(W_CRAT)    

        COMMON /GRID/ NROW,NCOL,XDIM,YDIM,YMAX,XMIN                     

C        COMMON /MODEL/RSEED,RANG(5),CNUG(5)                             
         COMMON /MODEL/RSEED,RANG(W_RANG),CNUG(W_CNUG)                             

C        COMMON /HERM/ COEF(10,5)                                        
         COMMON /HERM/ COEF(W_COEF1,W_COEF2)                                        

        COMMON /OPT/  NSTOP,KTEST,IKRIG,ITRANS                          

C        COMMON /HIST/ BIN(10)
         COMMON /HIST/ BIN(W_BIN)
                              
C        COMMON /TEST/ TX(20),TY(20)                                     
         COMMON /TEST/ TX(W_TX),TY(W_TY)                                     

C        COMMON /LTRAN/LVAR(5),LMEAN(5)
         COMMON /LTRAN/LVAR(W_LVAR), LMEAN(W_LMEAN)

        COMMON /PARM/ MVAR                                              

        DIMENSION BUF(500),RDOM(500)                                    

        REAL ICOS,LVAR,LMEAN                                            
        INTEGER BIN                                                     
        DOUBLE PRECISION RSEED,BSEED                                    


C Variables added by GJR.
        LOGICAL*1       SAVEUN, SAVECON, LOGBACK
        INTEGER*2       NREALZ
        CHARACTER*50    INFILE, UNCONSAVE, CONSAVE

C Open default output file. GJR.
        OPEN (6, FILE = 'COSIM.OUT')

        CALL HEADER

        CALL XYTAB (9,1)
        WRITE (*,*)' Enter input file name ==>'
        read  (*,231) INFILE
231     FORMAT (A30)

        CALL TIMER (IT1)
C
CGJR First read that initializes NREALZ.
C
        OPEN (22, FILE = INFILE)
         CALL INIT(KOUNT,KASTLE,NREALZ,SAVEUN,SAVECON,
     1                  UNCONSAVE, CONSAVE, LOGBACK )
        CLOSE (22)

C
CGJR Open up MODFLOW save files if needed as standard unformatted file.
C
        IF ( SAVEUN ) THEN
          OPEN (50, FILE = UNCONSAVE, ACCESS = 'TRANSPARENT')
        END IF

        IF ( SAVECON .AND. NSTOP .EQ. 1) THEN
        WRITE(*,*)'SAVE CON'
          OPEN (51, FILE = CONSAVE, ACCESS = 'TRANSPARENT')
        END IF

C
CGJR Begin DO loop that drives whole thing.
C
        DO 9000 IIC = 1, NREALZ

C...                                                                    
C.......DEFINE THE 15 VECTORS OF THE ICOSOHEDRON FOR SIMULATION         
C...                                                                    
        CALL VECT                                                       
C...                                                                    
C.......PROGRAM INITIALIZATION                                          
C...                                                                    

        IF (IIC .GT. 1) THEN
          OPEN (22, FILE = INFILE)
          CALL INIT(KOUNT,KASTLE,NREALZ,SAVEUN,SAVECON,
     1                  UNCONSAVE, CONSAVE, LOGBACK)
          CLOSE (22)
        END IF

C Increment seed by 2 each time, make sure to start at an odd number.
        RSEED = RSEED + FLOAT(IIC)*2.

        CALL HEADER
        CALL BOX(8,10,2,78)

        CALL XYTAB(9,5)
        WRITE(*,*)' Generating Realization...'

        CALL XYTAB(9,32)
        WRITE(*,'(I4)') IIC

        CALL XYTAB(9,40)
        WRITE(*,*)'of '

        CALL XYTAB (9,45)
        WRITE (*,'(I4)') NREALZ


C...                                                                    
C.......COMPUTE NGMAX: (JOURNEL AND HUIJBREGTS, 1978, P. 540).          
C...                                                                    
        BSEED      =  RSEED                                             
        IC         =  NROW * NROW                                       
        ID         =  NCOL * NCOL                                       
        XE         =  FLOAT(IC + ID)                                    
        NGMAX      =  SQRT(XE)   +  5                                   
C...                                                                    
C.......PROGRAM COSIM PRESENTLY YIELDS A SPHERICAL SPATIAL MODEL.       
C...
987     FORMAT(A30)

C Explicitly open scratch file for working storage.  GJR.
        OPEN (1, FILE = 'temp', FORM = 'UNFORMATTED')

        DO 3000 IVAR = 1,MVAR                                           
        REWIND 1                                                        
        NR         =  IFIX( RANG(IVAR) / ( 2.0 * XDIM ))                
        KD         =  2 * NR  +  1                                      
        XNR        =  FLOAT(NR)                                         
        C9         =  SQRT(36.0/(XNR * (XNR+1.0) * (2.0*XNR+1.0)))      
        NMORE      =  NGMAX + KD                                        
        RMIN       =  73057.00                                          
        RMAX       = -73057.00                                          
        DO 4  I    =  1,10                                              
        BIN(I)     =  0                                                 
4       CONTINUE                                                        
C...                                                                    
C.......COMPUTE 15 * NMORE RANDOM MEASURES.                             
C...                                                                    
        RMEAN      =  0.0                                               
        DO 100 IV  =  1,15                                              
        DO 5    I  =  1,500                                             
        RDOM(I)    =  0.0                                               
        BUF(I)     =  0.0                                               
5       CONTINUE                                                        
C...                                                                    
C.......COMPUTE NMORE RANDOM VARIABLES PER VECTOR.  RANDOM NUMBER       
C.......GENERATION IS PROVIDED BY FUNCTION DRAND.  THIS FUNCTION        
C.......RETURNS RANDOM NUMBERS IN THE RANGE 0 TO 1.  IN CONDITIONAL     
C.......SIMULATION, AN INITIAL DISTRIBUTION IS NEEDED HAVING A MEAN     
C.......OF ZERO.  DRAND RETURNS VALUES HAVING A MEAN OF 0.5; HENCE,     
C.......TO OBTAIN A MEAN OF ZERO, 0.5 IS SIMPLY SUBTRACTED FROM THE     
C.......VALUE, DRAND.                                                   
C...                                                                    
        DO 10 I    =  1,NMORE                                           
        RDOM(I)    =  DRAND(RSEED) - 0.50                               
        IF(RDOM(I).GT.RMAX) RMAX = RDOM(I)                              
        IF(RDOM(I).LT.RMIN) RMIN = RDOM(I)                              
        RMEAN      =  RMEAN + RDOM(I)                                   
10      CONTINUE                                                        
        XMN        = -0.50                                              
        XMAX       =  0.50                                              
        DO 30   I  =  1,NMORE                                           
        DO 30   J  =  1,10                                              
        XOLD       =  FLOAT(J - 1)*0.10 + XMN                           
        XNEW       =  FLOAT(J)    *0.10 + XMN                           
        IF(RDOM(I).GE.XOLD.AND.RDOM(I).LT.XNEW) BIN(J) = BIN(J) + 1     
30      CONTINUE                                                        
C...                                                                    
C.......COMPUTE NGMAX CORRELATED RANDOM VARIABLES FOR EACH OF THE       
C.......15 VECTORS.                                                     
C...                                                                    
        DO 40  J   =  1,NGMAX                                           
        KN         =  J + KD                                            
        M          =  0                                                 
        DO 40  I   =  J,KN                                              
        M          =  M + 1                                             
        DIFF       =  M - NR - 1                                        
        BUF(J)     =  BUF(J) + (DIFF * RDOM(I) * C9)                    
40      CONTINUE                                                        
        WRITE(1) (BUF(JK),JK = 1,NGMAX)                                 
100     CONTINUE                                                        
        WRITE(6,120)                                                    
120     FORMAT(1H1,T30,'HISTOGRAM OF RANDOM NUMBERS',//)                
        CALL HGRAM(RMIN,RMAX)                                           
C...                                                                    
C.......USING THE CORRELATED ONE DIMENSIONAL RANDOM VARIABLES           
C.......STORED ON UNIT 1, FORM THE 3-D SIMULATION.  (JOURNEL            
C.......AND HUIJBREGTS, 1978, P. 499).                                  
C...                                                                    

C        WRITE(*,*)' Performing unconditional simulation...'

        XBEGIN     =  - FLOAT(( NCOL + 1) / 2)                          
        YBEGIN     =    FLOAT(( NROW + 1) / 2)                          
        NVERT      =  1                                                 
        DO 150  I  =  1,NROW                                            
        DO 150  J  =  1,NCOL                                            
        RF(I,J,IVAR)  =  0.0                                            
150     CONTINUE                                                        
        REWIND 1                                                        
        SQ15       =  (1.0 / SQRT(15.0))                                
        LOCMIN     =  73057                                             
        LOCMAX     = -73057                                             
        DO 200 IRF =  1,15                                              
        READ(1) (BUF(JK),JK = 1,NGMAX)                                  
        DO 190  M  =  1,NVERT                                           
        DO 190  I  =  1,NROW                                            
        DO 190  J  =  1,NCOL                                            
        XNOW       =  XBEGIN + FLOAT(J)                                 
        YNOW       =  YBEGIN - FLOAT(I)                                 
        ZNOW       =  1.0                                               
        DUM1       =  ICOS(1,IRF)*XNOW + ICOS(2,IRF)*YNOW +             
     1                ICOS(3,IRF)*ZNOW + 0.5                            
        LOC        =  DUM1/2.0 + NGMAX/2                                
        IF(LOC.GT.LOCMAX) LOCMAX = LOC                                  
        IF(LOC.LT.LOCMIN) LOCMIN = LOC                                  
        IF(LOC.LE.0) GO TO 190                                          
        RF(I,J,IVAR) = RF(I,J,IVAR) + BUF(LOC) * SQ15                   
190     CONTINUE                                                        
200     CONTINUE                                                        
        WRITE(6,210) LOCMIN,LOCMAX                                      
210     FORMAT(/,T5,'LOCATION INDEX RANGE = ',I10,' TO ',I10,//)        
        RMEAN      =  RMEAN / FLOAT(15*NMORE)                           
        CALL PRT1(NGMAX,NR,KD,C9,BSEED,RMEAN,IVAR)                      
        CALL PRT2(IVAR)                                                 
        CALL VARG(XMEAN,VAR,IVAR)                                       
        CALL TRANS(XMEAN,VAR,IVAR)                                      
        CALL PRT2(IVAR)                                                 
        CALL VARG(XMEAN,VAR,IVAR)                                       
C...                                                                    
C.......RETURN FOR OTHER VARIABLES                                      
C...                                                                    
3000    CONTINUE                                                        
C...                                                                    
C.......CONDITION THE MODEL IF DESIRED.                                 
C...                                                                    

C Write unconditional results if desired. GJR.FIX
        IF (SAVEUN) CALL WRITE_UNF (RF, W_RFX, W_RFY, W_RFZ, 
     1                  NCOL, NROW, INT4(50), LOGBACK)

C        IF (SAVEUN) CALL WRITE_XYZ 

        IF(NSTOP.EQ.0) GO TO 3350                                       

C        WRITE (*,*)' Conditioning simulation....'
        CALL COKRIG(KOUNT,DET)                                          

C Write conditional simulation results. Added by GJR.

        IF (SAVECON) CALL WRITE_UNF (RF, W_RFX, W_RFY, W_RFZ,
     1                  NCOL, NROW, INT4(51), LOGBACK)


c        IF (SAVECON) CALL WRITE_XYZ 

        IF(DET.EQ.0.0) GO TO 5000                                       
C...                                                                    
C.......PRINT MODEL AND STATISTICAL ASPECTS.                            
C...                                                                    
        CALL PRT3(KOUNT)                                                
        DO 3100 I  =  1,MVAR                                            
        CALL PRT2(I)                                                    
        CALL VARG(XMEAN,VAR,I)                                          
3100    CONTINUE                                                        
        IF(KTEST.GT.0) CALL PTEST(KASTLE)                               
C...                                                                    
C.......CHECK CROSS-CORRELATION RESULTS.                                
C...                                                                    
        DO 3300 I  =  1,MVAR                                            
        DO 3300 J  =  1,MVAR                                            
        IF(J.LE.I) GO TO 3300                                           
        DO 3150 K  =  1,NROW                                            
        DO 3150 L  =  1,NCOL                                            
        RF(K,L,6)  =  RF(K,L,I) + RF(K,L,J)                             
3150    CONTINUE                                                        
        CALL VARG(XMEAN,VAR,6)                                          
3300    CONTINUE                                                        
3350    CONTINUE                                                        
C...                                                                    
C.......AS A FINAL CHECK, PRINT VECTORS OF ICOSOHEDRON.                 
C...                                                                    
        WRITE(6,3500)                                                   
        DO 3400 I  =  1,3                                               
        WRITE(6,4500)(ICOS(I,JK),JK = 1,15)                             
3400    CONTINUE                                                        
3500    FORMAT(1H1,T30,'CHECK ON 15 VECTORS',//)                        
4500    FORMAT(T3,15F5.2)                                               
        CALL PROPT                                                      
5000    CONTINUE                                                        

        CLOSE(1, STATUS='DELETE')       !random field scratch file

9000    CONTINUE

        IF (SAVEUN) CLOSE (50)                      !Input data file
        IF (SAVECON) CLOSE (51)                      !XYZ output file
        CLOSE (6)                       !General output file.
        CALL TIMER (IT2)
        TOT = IT2 - IT1
        TOT = TOT/100.
        CALL XYTAB (12,1)
        WRITE(*,11) TOT
11      FORMAT('  Elapsed time = ',f12.2,' seconds.')
        STOP                                                            
        END                                                             
C                                                                       
C                                                                       
        SUBROUTINE AFORM(KOUNT)                                         
        INCLUDE 'SIZES.INC'

C        COMMON /DAT2/ X(100),Y(100),DAT(100,5)                          
        COMMON /DAT2/ X(W_X),Y(W_Y),DAT(W_DAT1,W_DAT2)                          

        COMMON /PARM/ MVAR                                              

C        COMMON /VAR / CO(W_CO),C(W_C),RANGE(5),ANIS(5),RATIO(5)              
        COMMON /VAR / CO(W_CO),C(W_C),RANGE(W_RNG),ANIS(W_AN),
     1                  RATIO(W_RAT)

C        COMMON /CVAR/ CCO(10),CC(10),CRANGE(10),CANIS(10),CRATIO(10)    
        COMMON /CVAR/ CCO(W_CCO),CC(W_CC),CRANGE(W_CRAN),
     1                  CANIS(W_CAN),CRATIO(W_CRAT)    

C        COMMON /FORM/ A(100,100)                                        
        COMMON /FORM/ A(W_AX,W_AY)                                        

        COMMON /OPT/  NSTOP,KTEST,IKRIG,ITRANS                          
C...                                                                    
C.......THIS SUBROUTINE FORMS THE INTERSAMPLE COVARIANCE/CROSS-         
C.......COVARIANCE MATRIX, A, FOR ALL DATA TO WHICH THE NON-            
C.......CONDITIONAL SIMULATION IS TO BE CONDITIONED.                    
C...                                                                    
        DO 1000 II =  1,KOUNT                                           
        DO 750  JJ =  1,KOUNT                                           
        KPOS       =  0                                                 
        DO 500  KK =  1,MVAR                                            
        KTOT       =  (II - 1) * MVAR + KK                              
        LTOT       =  (JJ - 1) * MVAR + KK                              
        DO 500  LL =  1,MVAR                                            
        NTOT       =  (JJ - 1) * MVAR + LL                              
        MTOT       =  (II - 1) * MVAR + LL                              
        DIFX       =  X(II) - X(JJ)                                     
        DIFY       =  Y(II) - Y(JJ)                                     
        IF(LL.NE.KK)  GO TO 100                                         
        DISTAN     =  SQRT((DIFX * COS(ANIS(LL)) + DIFY * SIN(ANIS      
     1                (LL)))**2 + (RATIO(LL) * (DIFY * COS(ANIS(LL))    
     2                - DIFX * SIN(ANIS(LL))))**2)                      
        I9         =  LL                                                
        A(KTOT,NTOT) = COVAR(DISTAN,I9)                                 
        GO TO 500                                                       
100     CONTINUE                                                        
        IF(LL.LT.KK) GO TO 500                                          
        KPOS       =  KPOS + 1                                          
        DISTAN     =  SQRT((DIFX * COS(CANIS(KPOS)) + DIFY * SIN        
     1                (CANIS(KPOS)))**2 + (CRATIO(KPOS) * (DIFY *       
     2                COS(CANIS(KPOS)) - DIFX * SIN(CANIS(KPOS))))      
     3                **2)                                              
        I8         =  KK                                                
        I9         =  LL                                                
        A(KTOT,NTOT) = CROSS(DISTAN,KPOS,I8,I9)                         
        A(MTOT,LTOT) = A(KTOT,NTOT)                                     
500     CONTINUE                                                        
750     CONTINUE                                                        
C...                                                                    
C.......LAST MATRIX IN EACH ROW OF MATRIX, A, IS A UNIT MATRIX.         
C...                                                                    
        DO 900 MM  =  1,MVAR                                            
        IA         =  (II - 1) * MVAR + MM                              
        DO 900 NN  =  1,MVAR                                            
        KM         =  KOUNT * MVAR + NN                                 
        IF(MM.EQ.NN)  A(IA,KM) = 1.0                                    
        IF(MM.NE.NN)  A(IA,KM) = 0.0                                    
900     CONTINUE                                                        
1000    CONTINUE                                                        
C...                                                                    
C.......FORM LAST ROW OF A:  A ROW OF UNIT AND NULL MATRICES.           
C...                                                                    
        DO 1200 LM =  1,KOUNT                                           
        DO 1200 MM =  1,MVAR                                            
        IA         =  KOUNT * MVAR + MM                                 
        DO 1200 NN =  1,MVAR                                            
        NTOT       =  (LM - 1) * MVAR + NN                              
        IF(MM.EQ.NN)  A(IA,NTOT) = 1.0                                  
        IF(MM.NE.NN)  A(IA,NTOT) = 0.0                                  
1200    CONTINUE                                                        
C...                                                                    
C.......LAST MATRIX IN THIS ROW IS NULL                                 
C...                                                                    
        DO 1400 MM =  1,MVAR                                            
        IA         =  KOUNT * MVAR + MM                                 
        DO 1400 NN =  1,MVAR                                            
        IB         =  KOUNT * MVAR + NN                                 
        A(IA,IB)   =  0.0                                               
1400    CONTINUE                                                        
C...                                                                    
C.......ACCOUNT FOR UNDERSAMPLED RANDOM FUNCTIONS IF DESIRED.           
C...                                                                    
        IF(IKRIG.EQ.1) CALL UNSAM(KOUNT)                                
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        SUBROUTINE COKRIG(KOUNT,DET)                                    
        INCLUDE 'SIZES.INC'

C        COMMON /SIM/  RF(65,65,6)                                       
         COMMON /SIM/ RF(W_RFX,W_RFY,W_RFZ)                                        

C        COMMON /DAT2/ X(100),Y(100),DAT(100,5)                          
         COMMON /DAT2/ X(W_X),Y(W_Y),DAT(W_DAT1,W_DAT2)                          

        COMMON /PARM/ MVAR                                              
        COMMON /OPT/  NSTOP,KTEST,IKRIG,ITRANS                          
        COMMON /GRID/ NROW,NCOL,XDIM,YDIM,YMAX,XMIN                     

C        COMMON /FORM/ A(100,100)                                        
         COMMON /FORM/ A(W_AX,W_AY)                                        

C        COMMON /VAR/  CO(5),C(5),RANGE(5),ANIS(5),RATIO(5)              
         COMMON /VAR/ CO(W_CO),C(W_C),RANGE(W_RNG),
     1                  ANIS(W_AN),RATIO(W_RAT)

C        COMMON /CVAR/ CCO(10),CC(10),CRANGE(10),CANIS(10),CRATIO(10)    
         COMMON /CVAR/ CCO(W_CCO),CC(W_CC),CRANGE(W_CRAN),
     1                  CANIS(W_CAN),CRATIO(W_CRAT)    

        DIMENSION XMEAS(100,5),WEIGHT(100,5),SIMU(100,5)                
        DIMENSION TEMP(5,5),TEMP1(5,5),TEMP2(5,5),EST(5,2)              
C...                                                                    
C.......THIS SUBROUTINE CONTROLS THE CONDITIONING, USING COKRIGING,     
C.......OF THE NON-CONDITIONAL SIMULATION TO THE SAMPLE DATA.           
C.......                                                                
C.......STEP 1.  FIND SIMULATED VALUES (SIMU) CLOSEST                   
C.......         TO THE ACTUAL DATA LOCATIONS.                          
C...                                                                    
        YBEGIN     =  YMAX + 0.5 * YDIM                                 
        XBEGIN     =  XMIN - 0.5 * XDIM                                 
        DO 10  I   =  1,KOUNT                                           
        IROW       =  (YBEGIN - Y(I)) / YDIM                            
        JCOL       =  (X(I) - XBEGIN) / XDIM                            
        IF(IROW.LE.0) IROW = 1                                          
        IF(IROW.GT.NROW) IROW = NROW                                    
        IF(JCOL.LE.0) JCOL = 1                                          
        IF(JCOL.GT.NCOL) JCOL = NCOL                                    
        DO 10  J   =  1,MVAR                                            
        SIMU(I,J)  =  RF(IROW,JCOL,J)                                   
10      CONTINUE                                                        
C...                                                                    
C.......STEP 2.  FORM THE INTERSAMPLE COVARIANCE/CROSS-COVARIANCE       
C.......         MATRIX, A.                                             
C...                                                                    
        CALL AFORM(KOUNT)                                               
C...                                                                    
C.......STEP 3.  INVERT THE MATRIX, A.                                  
C...                                                                    
        NN         =  KOUNT + 1                                         
        CALL EQSOLV(NN,DET)                                             
        IF(DET.EQ.0.0) WRITE(6,30)                                      
30      FORMAT(/,'A-MATRIX IS SINGULAR',//)                             
        IF(DET.EQ.0.0) RETURN                                           
C...                                                                    
C.......STEP 4.  USING A-INV, CONDITION THE MODEL.                      
C...                                                                    
        DO 1000 II =  1,NROW                                            
        YCORD      =  YBEGIN - FLOAT(II) * YDIM                         
        DO 1000 JJ =  1,NCOL                                            
        XCORD      =  XBEGIN + FLOAT(JJ) * XDIM                         
C...                                                                    
C.......STEP 4A.  FORM THE MEASUREMENT VECTOR FOR LOCATION (II,JJ)      
C...                                                                    
        DO 500  KK =  1,KOUNT                                           
        DIFX       =  X(KK) - XCORD                                     
        DIFY       =  Y(KK) - YCORD                                     
        IF(IKRIG.EQ.0) GO TO 50                                         
        IF(DIFX.EQ.0.0.AND.DIFY.EQ.0.0) DIFX = 0.20                     
50      CONTINUE                                                        
        KPOS       =  0                                                 
        DO 200  I  =  1,MVAR                                            
        KZ         =  (KK - 1) * MVAR + I                               
        DO 200  J  =  1,MVAR                                            
        KW         =  (KK - 1) * MVAR + J                               
        IF(J.NE.I)    GO TO 100                                         
        DISTAN     =  SQRT((DIFX * COS(ANIS(J)) + DIFY * SIN(ANIS       
     1                (J)))**2 + (RATIO(J)*(DIFY*COS(ANIS(J)) -         
     2                DIFX * SIN(ANIS(J))))**2)                         
        I9         =  J                                                 
        XMEAS(KZ,J) =  COVAR(DISTAN,I9)                                 
        GO TO 200                                                       
100     CONTINUE                                                        
        IF(J.LT.I)    GO TO 200                                         
        KPOS       =  KPOS + 1                                          
        DISTAN     =  SQRT((DIFX * COS(CANIS(KPOS)) + DIFY * SIN        
     1                (CANIS(KPOS)))**2 + (CRATIO(KPOS) * (DIFY *       
     2                COS(CANIS(KPOS)) - DIFX * SIN(CANIS(KPOS))))      
     3                **2)                                              
        I8         =  I                                                 
        I9         =  J                                                 
        XMEAS(KZ,J) =  CROSS(DISTAN,KPOS,I8,I9)                         
        XMEAS(KW,I) =  XMEAS(KZ,J)                                      
200     CONTINUE                                                        
500     CONTINUE                                                        
C...                                                                    
C.......LAST MATRIX IS A UNIT MATRIX                                    
C...                                                                    
        DO 600  I  =  1,MVAR                                            
        KZ         =  KOUNT * MVAR + I                                  
        DO 600  J  =  1,MVAR                                            
        IF(I.EQ.J)    XMEAS(KZ,J) = 1.0                                 
        IF(I.NE.J)    XMEAS(KZ,J) = 0.0                                 
600     CONTINUE                                                        
C...                                                                    
C.......MODIFY THIS VECTOR FOR UNDERSAMPLING IF DESIRED.                
C...                                                                    
        IF(IKRIG.EQ.1) CALL UNVEC(XMEAS,KOUNT)                          
C...                                                                    
C.......STEP 4B.  FORM THE WEIGHTING VECTOR FOR LOCATION (II,JJ).       
C...                                                                    
        DO 650 I   =  1,NN                                              
        DO 650 J   =  1,MVAR                                            
        KZ         =  (I - 1) * MVAR + J                                
        DO 650 K   =  1,MVAR                                            
        WEIGHT(KZ,K)  =  0.0                                            
650     CONTINUE                                                        
        DO 800  I  =  1,NN                                              
        DO 800  J  =  1,NN                                              
        DO 700  K  =  1,MVAR                                            
        KZ         =  (I - 1) * MVAR + K                                
        KT         =  (J - 1) * MVAR + K                                
        DO 700  L  =  1,MVAR                                            
        KW         =  (J - 1) * MVAR + L                                
        TEMP(K,L)  =  A(KZ,KW)                                          
        TEMP1(K,L) =  XMEAS(KT,L)                                       
700     CONTINUE                                                        
        CALL MATMUL(TEMP,TEMP1,TEMP2,MVAR,MVAR,MVAR)                    
        DO 800  K  =  1,MVAR                                            
        KZ         =  (I - 1) * MVAR + K                                
        DO 800  L  =  1,MVAR                                            
        WEIGHT(KZ,L)  =  WEIGHT(KZ,L) + TEMP2(K,L)                      
800     CONTINUE                                                        
C...                                                                    
C.......STEP 4C.  USING THE WEIGHTING VECTOR, COMPUTE KRIGED EST.       
C...                                                                    
        DO 810 I   =  1,MVAR                                            
        EST(I,1)   =  0.0                                               
        EST(I,2)   =  0.0                                               
810     CONTINUE                                                        
        DO 900  I  =  1,KOUNT                                           
        DO 900  J  =  1,MVAR                                            
        DO 900  K  =  1,MVAR                                            
        KZ         =  (I - 1) * MVAR + K                                
        IF(IKRIG.EQ.0) GO TO 890                                        
        IF(DAT(I,K).EQ.0.0) SIMU(I,K) = 0.0                             
890     CONTINUE                                                        
        EST(J,1)   =  EST(J,1) + DAT(I,K) * WEIGHT(KZ,J)                
        EST(J,2)   =  EST(J,2) + SIMU(I,K) * WEIGHT(KZ,J)               
900     CONTINUE                                                        
C...                                                                    
C.......STEP 4D.  CONDITION THE MODEL: RF = RF - SIM.EST + DAT.EST.     
C...                                                                    
        DO 950  I  =  1,MVAR                                            
        RF(II,JJ,I) = RF(II,JJ,I) - EST(I,2) + EST(I,1)                 
950     CONTINUE                                                        
1000    CONTINUE                                                        
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        FUNCTION COVAR(DIST,K)                                          
        INCLUDE 'SIZES.INC'

C        COMMON /VAR/ CO(5),C(5),RANGE(5),ANIS(5),RATIO(5)               
         COMMON /VAR/ CO(W_CO),C(W_C),RANGE(W_RNG),
     1                  ANIS(W_AN),RATIO(W_RAT)

C...                                                                    
C.......THIS FUNCTION EVALUATES THE MODEL COVARIANCE ASSOCIATED         
C.......WITH THE SEPARATION DISTANCE, DIST.                             
C...                                                                    
        IF(DIST.GE.RANGE(K)) GO TO 120                                  
        B          =  C(K) - CO(K)                                      
        DUM1       =  CO(K) + B*(1.5*DIST/RANGE(K) - 0.5*(DIST/         
     1                RANGE(K))**3)                                     
        IF(DIST.EQ.0.0) DUM1 = 0.0                                      
        COVAR      =  C(K) - DUM1                                       
        RETURN                                                          
120     CONTINUE                                                        
        COVAR      =  0.0                                               
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        FUNCTION CROSS(DIST,K,IA,IB)                                    
        INCLUDE 'SIZES.INC'

C        COMMON /CVAR/ CCO(10),CC(10),CRANGE(10),CANIS(10),CRATIO(10)    
         COMMON /CVAR/ CCO(W_CCO),CC(W_CC),CRANGE(W_CRAN),
     1                  CANIS(W_CAN),CRATIO(W_CRAT)    

C...                                                                    
C.......THIS FUNCTION EVALUATES THE CROSS-COVARIANCE VALUE              
C.......CORRESPONDING TO THE DISTANCE, DIST.                            
C...                                                                    
        IF(DIST.GE.CRANGE(K)) GO TO 120                                 
        B          =  CC(K) - CCO(K)                                    
        DUM1       =  CCO(K) + B*(1.5*DIST/CRANGE(K)                    
     1                - 0.5*(DIST/CRANGE(K))**3)                        
        IF(DIST.EQ.0.0) DUM1 = 0.0                                      
        DUM2       =  CC(K) - DUM1                                      
        GO TO 400                                                       
120     CONTINUE                                                        
        DUM2       =  0.0                                               
400     CONTINUE                                                        
        DUM3       =  COVAR(DIST,IA)                                    
        DUM4       =  COVAR(DIST,IB)                                    
        CROSS      =  0.5 * (DUM2 - DUM3 - DUM4)                        
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        FUNCTION DRAND(IX)                                              
C...                                                                    
C.......THIS IS A PORTABLE RANDOM NUMBER GENERATION                     
C.......FUNCTION.  REFERENCE:                                           
C.......                                                                
C.......           SCHRAGE,L.,"A MORE PORTABLE FORTRAN RANDOM           
C.......             NUMBER GENERATOR," ACM TRANSACTIONS ON             
C.......             MATHEMATICAL SOFTWARE, VOL.5, NO.2, JUNE, 1979.    
C.......                                                                
C.......IX = IX * A MOD P IS THE RECURSION USED.                        
C...                                                                    
        DOUBLE PRECISION A,P,IX,B15,B16,XHI,XALO,LEFTLO,FHI,K           
        A          =  16807.D0                                          
        B15        =  32768.D0                                          
        B16        =  65536.D0                                          
        P          =  2147483647.D0                                     
C...                                                                    
C.......GET 15 HIGH ORDER BITS OF IX                                    
C...                                                                    
        XHI        =  IX / B16                                          
        XHI        =  XHI - DMOD(XHI,1.D0)                              
C...                                                                    
C.......GET 16 LOW BITS OF IX AND FORM LOW PRODUCT                      
C...                                                                    
        XALO       =  (IX - XHI * B16) * A                              
C...                                                                    
C.......GET 15 HIGH ORDER BITS OF LOW PRODUCT                           
C...                                                                    
        LEFTLO     =  XALO / B16                                        
        LEFTLO     =  LEFTLO - DMOD(LEFTLO,1.D0)                        
C...                                                                    
C.......FORM THE 31 HIGHEST BITS OF FULL PRODUCT                        
C...                                                                    
        FHI        =  XHI * A  +  LEFTLO                                
C...                                                                    
C.......GET OVERFLOW PAST 31ST BIT OF FULL PRODUCT                      
C...                                                                    
        K          =  FHI / B15                                         
        K          =  K - DMOD(K,1.D0)                                  
C...                                                                    
C.......ASSEMBLE ALL THE PARTS AND PRESUBTRACT P.  THE PARENTHESES      
C.......ARE ESSENTIAL.                                                  
C...                                                                    
        IX         =  (((XALO - LEFTLO*B16) - P) + (FHI - K*B15)        
     1                * B16) + K                                        
C...                                                                    
C.......ADD P BACK IF NECESSARY                                         
C...                                                                    
        IF(IX.LT.0.D0) IX = IX + P                                      
C...                                                                    
C.......MULTIPLY BY (1 /  (2**31 - 1))                                  
C...                                                                    
        DRAND      = IX  * 4.656612875D-10                              
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        SUBROUTINE EQSOLV(N,DET)                                        

        INCLUDE 'SIZES.INC'
C...                                                                    
C.......THIS SUBROUTINE COMPUTES THE INVERSE OF THE INTERSAMPLE         
C.......COVARIANCE/CROSS-COVARIANCE MATRIX, A.  THIS IS A               
C.......MODIFIED VERSION OF SUBROUTINE, CHAP8, PRESENTED IN:            
C.......      KUO, SHAN S., "NUMERICAL METHODS AND COMPUTERS,"          
C.......        ADDISON-WESLEY PUBLISHING, READING, MASSACHUSETTS,      
C.......        USA, 1965, PP. 168-169.                                 
C.......                                                                
C.......        A:  COVARIANCE MATRIX AND ITS INVERSE                   
C.......        N:  ORDER (DIMENSION) OF A                              
C.......      DET:  DETERMINANT OF A                                    
C...                                                                    
        COMMON /PARM/ MVAR                                              

C        COMMON /FORM/ A(100,100)                                        
         COMMON /FORM/ A(W_AX,W_AY)                                        

        COMMON /OPT/  NSTOP,KTEST,IKRIG,ITRANS                          

        DIMENSION IPIVOT(100),INDEX(100,2),PIVOT(100,5)                 
        DIMENSION TEMP(5,5),TEMP1(5,5),TEMP2(5,5)                       
C...                                                                    
C.......INITIALIZE THE INVERSION OF A.                                  
C...                                                                    
        DET        =  1.0                                               
        DO 10  J   =  1,N                                               
        IPIVOT(J)  =  0                                                 
10      CONTINUE                                                        
        DO 1350 I  =  1,N                                               
C...                                                                    
C.......THE FOLLOWING STATEMENTS SEARCH FOR PIVOT ELEMENT.              
C...                                                                    
        T          =  0.0                                               
        DO 90  J   =  1,N                                               
        IF(IPIVOT(J) - 1)  15,90,15                                     
15      DO 80  K   =  1,N                                               
        IF(IPIVOT(K) - 1)  40,80,8100                                   
40      CONTINUE                                                        
        DO 50 JJ   =  1,MVAR                                            
        JL         =  (J - 1) * MVAR + JJ                               
        DO 50 KK   =  1,MVAR                                            
        JM         =  (K - 1) * MVAR + KK                               
        TEMP(JJ,KK)   =  A(JL,JM)                                       
50      CONTINUE                                                        
        IF(IKRIG.EQ.0.OR.I.EQ.1) GO TO 59                               
        IF(J - K) 80,55,80                                              
55      IROW       =  J                                                 
        ICOL       =  K                                                 
        GO TO 80                                                        
59      CONTINUE                                                        
        W          =  TRACE(TEMP)                                       
        IF(ABS(T) - ABS(W)) 60,80,80                                    
60      IROW       =  J                                                 
        ICOL       =  K                                                 
        T          =  W                                                 
80      CONTINUE                                                        
90      CONTINUE                                                        
        IPIVOT(ICOL) = IPIVOT(ICOL) + 1                                 
C...                                                                    
C.......THE FOLLOWING STATEMENTS PUT THE PIVOT ELEMENT ON DIAGONAL.     
C...                                                                    
        IF(IROW - ICOL)  100,150,100                                    
100     DET          =  -DET                                            
        DO 130  L    =  1,N                                             
        DO 120  LI   =  1,MVAR                                          
        LL           =  (IROW - 1) * MVAR + LI                          
        DO 120  LJ   =  1,MVAR                                          
        LM           =  (L - 1) * MVAR + LJ                             
        TEMP(LI,LJ)  =  A(LL,LM)                                        
120     CONTINUE                                                        
        DO 130  LI   =  1,MVAR                                          
        LK           =  (IROW - 1) * MVAR + LI                          
        LL           =  (ICOL - 1) * MVAR + LI                          
        DO 130  LJ   =  1,MVAR                                          
        LM           =  (L - 1) * MVAR + LJ                             
        A(LK,LM)     =  A(LL,LM)                                        
        A(LL,LM)     =  TEMP(LI,LJ)                                     
130     CONTINUE                                                        
150     CONTINUE                                                        
        INDEX(I,1)   =  IROW                                            
        INDEX(I,2)   =  ICOL                                            
        DO 160  LI   =  1,MVAR                                          
        IP           =  (I - 1) * MVAR + LI                             
        IA           =  (ICOL - 1) * MVAR + LI                          
        DO 160  LJ   =  1,MVAR                                          
        IB           =  (ICOL - 1) * MVAR + LJ                          
        PIVOT(IP,LJ) =  A(IA,IB)                                        
        TEMP(LI,LJ)  =  PIVOT(IP,LJ)                                    
160     CONTINUE                                                        
        W            =  TRACE(TEMP)                                     
        DET          =  DET * W                                         
C...                                                                    
C.......THE FOLLOWING STATEMENTS DIVIDE PIVOT ROW BY PIVOT ELEMENT.     
C...                                                                    
        DO 170  LI =  1,MVAR                                            
        IA         =  (ICOL - 1) * MVAR + LI                            
        DO 170  LJ =  1,MVAR                                            
        IB         =  (ICOL - 1) * MVAR + LJ                            
        IF(LI.EQ.LJ)  A(IA,IB) = 1.0                                    
        IF(LI.NE.LJ)  A(IA,IB) = 0.0                                    
170     CONTINUE                                                        
        DO 200  L  =  1,N                                               
        DO 180 LI  =  1,MVAR                                            
        IP         =  (I - 1) * MVAR + LI                               
        IA         =  (ICOL - 1) * MVAR + LI                            
        DO 180 LJ  =  1,MVAR                                            
        IB         =  (L - 1) * MVAR + LJ                               
        TEMP(LI,LJ)   =  A(IA,IB)                                       
        TEMP1(LI,LJ)  =  PIVOT(IP,LJ)                                   
180     CONTINUE                                                        
        CALL SCALG(TEMP1,TEMP,TEMP2)                                    
        DO 200  LI =  1,MVAR                                            
        IA         =  (ICOL - 1) * MVAR + LI                            
        DO 200  LJ =  1,MVAR                                            
        IB         =  (L - 1) * MVAR + LJ                               
        A(IA,IB)   =  TEMP2(LI,LJ)                                      
200     CONTINUE                                                        
C...                                                                    
C.......REDUCE THE NON PIVOT ROWS.                                      
C...                                                                    
        DO 1350 LI =  1,N                                               
        IF(LI - ICOL)  210,1350,210                                     
210     CONTINUE                                                        
        DO 220  LJ =  1,MVAR                                            
        IA         =  (LI - 1) * MVAR + LJ                              
        DO 220  LM =  1,MVAR                                            
        IB         =  (ICOL - 1) * MVAR + LM                            
        TEMP(LJ,LM)   =  A(IA,IB)                                       
        A(IA,IB)      =  0.0                                            
220     CONTINUE                                                        
        DO 240  L  =  1,N                                               
        DO 230  LJ =  1,MVAR                                            
        IA         =  (ICOL - 1) * MVAR + LJ                            
        DO 230  LM =  1,MVAR                                            
        IB         =  (L - 1) * MVAR + LM                               
        TEMP1(LJ,LM)  =  A(IA,IB)                                       
230     CONTINUE                                                        
        CALL MATMUL(TEMP,TEMP1,TEMP2,MVAR,MVAR,MVAR)                    
        DO 240  LJ =  1,MVAR                                            
        IA         =  (LI - 1) * MVAR + LJ                              
        DO 240  LM =  1,MVAR                                            
        IB         =  (L - 1) * MVAR + LM                               
        A(IA,IB)   =  A(IA,IB) - TEMP2(LJ,LM)                           
240     CONTINUE                                                        
1350    CONTINUE                                                        
C...                                                                    
C.......THE FOLLOWING STATEMENTS INTERCHANGE COLUMNS.                   
C...                                                                    
        DO 3000 I  =  1,N                                               
        L          =  N - I + 1                                         
        IF(INDEX(L,1) - INDEX(L,2)) 2000,3000,2000                      
2000    JROW       =  INDEX(L,1)                                        
        JCOL       =  INDEX(L,2)                                        
        DO 2500 K  =  1,N                                               
        DO 2400 KI =  1,MVAR                                            
        IA         =  (K - 1) * MVAR + KI                               
        DO 2400 KJ =  1,MVAR                                            
        IB         =  (JROW - 1) * MVAR + KJ                            
        IC         =  (JCOL - 1) * MVAR + KJ                            
        TEMP(KI,KJ)   =  A(IA,IB)                                       
        A(IA,IB)      =  A(IA,IC)                                       
        A(IA,IC)      =  TEMP(KI,KJ)                                    
2400    CONTINUE                                                        
2500    CONTINUE                                                        
3000    CONTINUE                                                        
8100    RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        SUBROUTINE HGRAM(XMIN,XMAX)                                     
        INCLUDE 'SIZES.INC'

C        COMMON /HIST/ BIN(10)
         COMMON /HIST/ BIN(W_BIN)

        CHARACTER*1 FREQ(50)                                            
        DIMENSION IOBS(10),CEL(10),CUML(10)                             
        INTEGER BIN,BMAX                                                
        REAL INC,INT,CEL,CUML                                           
C...                                                                    
C.......THIS SUBROUTINE PLOTS A HISTOGRAM.                              
C...                                                                    
        ISUM       =  0                                                 
        DO 4   I   =  1,10                                              
        ISUM       =  ISUM + BIN(I)                                     
4       CONTINUE                                                        
        DO 5   I   =  1,10                                              
        IOBS(I)    =  BIN(I)                                            
        DUM1       =  FLOAT(BIN(I))                                     
        DUM2       =  FLOAT(ISUM)                                       
        CEL(I)     =  DUM1 / DUM2                                       
        CUML(I)    =  0.0                                               
        DO 5   J   =  1,I                                               
        CUML(I)    =  CUML(I) + CEL(J)                                  
5       CONTINUE                                                        
        DO 10 I    =  1,50                                              
        FREQ(I)    =  '*'                                               
10      CONTINUE                                                        
        BMAX       = -73057                                             
        DO 15   I  =  1,10                                              
        IF(BIN(I) .GT. BMAX) BMAX = BIN(I)                              
15      CONTINUE                                                        
        DO 16   I  =  1,10                                              
        BIN(I)     =  BIN(I) * 50/BMAX                                  
16      CONTINUE                                                        
        INC        =  (XMAX - XMIN)/10.0                                
        BEGIN      =  XMIN - INC                                        
        WRITE(6,17)                                                     
17      FORMAT(//,T5,'OBS',3X,'REL',2X,'CUML',3X,'LOW',/)               
        DO 20   I  =  1,10                                              
        INT        =  BEGIN + FLOAT(I) * INC                            
        J          =  BIN(I)                                            
        WRITE(6,30) IOBS(I),CEL(I),CUML(I),INT,(FREQ(JK),JK = 1,J)      
20      CONTINUE                                                        
        WRITE(6,25) ISUM                                                
25      FORMAT(/,T2,I6)                                                 
30      FORMAT(T2,I6,2F6.3,F6.1,2X,'+',1X,50A1)                         
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        SUBROUTINE INIT (KOUNT,KASTLE,NREALZ,SAVEUN,SAVECON,
     1                           UNCONSAVE, CONSAVE, LOGBACK)

C Variables added by GJR.
        LOGICAL*1       SAVEUN, SAVECON, LOGBACK
        CHARACTER*50    UNCONSAVE, CONSAVE
        INTEGER*2       NREALZ
C End of my stuff, GJR.

        INCLUDE 'SIZES.INC'

        COMMON /GRID/ NROW,NCOL,XDIM,YDIM,YMAX,XMIN

C        COMMON /MODEL/RSEED,RANG(5),CNUG(5)
         COMMON /MODEL/RSEED,RANG(W_RANG),CNUG(W_CNUG)                             

C        COMMON /HERM/ COEF(10,5)
         COMMON /HERM/ COEF(W_COEF1,W_COEF2)                                        

        COMMON /OPT/  NSTOP,KTEST,IKRIG,ITRANS

C        COMMON /TEST/ TX(20),TY(20)
         COMMON /TEST/ TX(W_TX),TY(W_TY)                                     

C        COMMON /VAR/  CO(5),C(5),RANGE(5),ANIS(5),RATIO(5)
         COMMON /VAR/ CO(W_CO),C(W_C),RANGE(W_RNG),
     1                  ANIS(W_AN),RATIO(W_RAT)

C        COMMON /CVAR/ CCO(10),CC(10),CRANGE(10),CANIS(10),CRATIO(10)
         COMMON /CVAR/ CCO(W_CCO),CC(W_CC),CRANGE(W_CRAN),
     1                  CANIS(W_CAN),CRATIO(W_CRAT)    

C        COMMON /DAT2/ X(100),Y(100),DAT(100,5)
         COMMON /DAT2/ X(W_X),Y(W_Y),DAT(W_DAT1,W_DAT2)                          

C        COMMON /LTRAN/LVAR(5),LMEAN(5)
         COMMON /LTRAN/LVAR(W_LVAR), LMEAN(W_LMEAN)

        COMMON /PARM/ MVAR

        DIMENSION DUM(5)
        REAL LVAR,LMEAN
        DOUBLE PRECISION RSEED
C...
C.......THIS SUBROUTINE ACCESSES USER SUPPLIED DATA.
C...
C GJR altered this subroutine to read from file on unit 22 (instead of 5).
        KOUNT      =  0
        NSTOP      =  0
        KTEST      =  0
        KASTLE     =  0

C        WRITE(*,*)' Reading input data....'

CGJR  My additions to input file.
        READ(22,*) NREALZ
        READ(22,*) SAVEUN, SAVECON
        READ(22,*) UNCONSAVE, CONSAVE
        READ(22,*) LOGBACK

C...
C.......1.  INPUT NUMBER OF RANDOM FUNCTIONS TO BE SIMULATED.
C...
        READ(22,*) MVAR
C...
C.......INPUT INITIAL CONSTANTS
C...
C.......2.  ESTABLISH GRID DIMENSIONS
C...
        READ(22,*) NROW,NCOL,XDIM,YDIM
C...
C.......3.  ESTABLISH GRID GEOGRAPHIC REGISTRATION
C...
        READ(22,*) YMAX,XMIN
C...
C.......4.  ESTABLISH SIMULATION SPATIAL PARAMETERS
C...
        READ(22,*) RSEED
C...
C.......5.  ESTABLISH MODEL TRANSFORMATION
C...
        READ(22,*) ITRANS

        IF (ITRANS.EQ.1) GO TO 15

        DO 10  I   =  1,MVAR
        READ(22,*) LVAR(I),LMEAN(I),RANG(I),CNUG(I)
10      CONTINUE

        GO TO 30

15      CONTINUE

        DO 20 I = 1,MVAR
        READ(22,*) (COEF(JK,I), JK = 1,10),RANG(I)
        CNUG(I)    =  0.0
20      CONTINUE
30      CONTINUE

C...
C.......6.  INPUT CONDITIONING OPTION
C...
        READ(22,*) NSTOP
C...
C.......7.  INPUT VARIOGRAM AND CROSS VARIOGRAM PARAMETERS.
C...
        IF(NSTOP.EQ.0) GO TO 1000

        DO 100  I  =  1,MVAR
        READ(22,*) CO(I),C(I),RANGE(I),ANIS(I),RATIO(I)
        ANIS(I)    =  ANIS(I) * 0.01745329
100     CONTINUE

        IF(MVAR.EQ.1) GO TO 160
        N          =  (MVAR * (MVAR - 1)) / 2

        DO 150  I  =  1,N
        READ(22,*) CCO(I),CC(I),CRANGE(I),CANIS(I),CRATIO(I)
        CANIS(I)   =  CANIS(I) * 0.01745329
150     CONTINUE

160     CONTINUE
C...
C.......8.  INPUT CONDITIONING DATA.
C...
        READ(22,*) IKRIG
200     CONTINUE
        READ(22,*) D1,D2,(DUM(JK),JK = 1,MVAR)
        IF (D1.EQ.0.0 .AND. D2.EQ.0.0) GO TO 500
        KOUNT      =  KOUNT + 1
        X(KOUNT)   =  D2
        Y(KOUNT)   =  D1
        DO 250  I  =  1,MVAR
        DAT(KOUNT,I)  =  DUM(I)
250     CONTINUE
        GO TO 200
C...
C.......9.  INPUT TEST LOCATIONS IF DESIRED.
C...
500     CONTINUE
        READ(22,*) KTEST

        IF(KTEST.EQ.0) GO TO 1000
600     CONTINUE
        READ(22,*) D1,D2
        IF(D1.EQ.0.0.AND.D2.EQ.0.0) GO TO 1000
        KASTLE     =  KASTLE + 1
        TX(KASTLE) =  D1
        TY(KASTLE) =  D2
        GO TO 600
1000    CONTINUE
        RETURN
        END
C                                                                       
C                                                                       
        SUBROUTINE MATML1(A,B,C,I,J,K)                                  
C...                                                                    
C.......THIS SUBROUTINE IS DESIGNED FOR THE MULTIPLICATION OF           
C.......TWO MATRICES, A AND B.  FORMULA:  A X B = C.  NOTE:             
C.......THE ORDER OF MULTIPLICATION SHOWN IS ESSENTIAL.                 
C...                                                                    
        DIMENSION A(3,3),B(3,3),C(3,3)                                  
        DO 10 LI   =  1,I                                               
        DO 10 LJ   =  1,J                                               
        C(LI,LJ)   =  0.0                                               
        DO 10 LK   =  1,K                                               
        C(LI,LJ)   =  C(LI,LJ) + A(LI,LK) * B(LK,LJ)                    
10      CONTINUE                                                        
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        SUBROUTINE MATMUL(A,B,C,J,K,L)                                  
        DIMENSION A(5,5),B(5,5),C(5,5)                                  
C...                                                                    
C.......THIS SUBROUTINE PERFORMS A GENERAL MATRIX                       
C.......MULTIPLICATION OF THE FORM:  C = A X B.                         
C...                                                                    
        DO 100 I   =  1,J                                               
        DO 100 M   =  1,L                                               
        C(I,M)     =  0.0                                               
        DO 100 N   =  1,K                                               
        C(I,M)     =  C(I,M) + A(I,N) * B(N,M)                          
100     CONTINUE                                                        
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        SUBROUTINE NUGGET(K)                                            
        INCLUDE 'SIZES.INC'

C...                                                                    
C.......THIS SUBROUTINE ADDS A NUGGET EFFECT TO THE MODEL.              
C...                                                                    
C        COMMON /MODEL/ RSEED,RANG(5),CNUG(5)                            
         COMMON /MODEL/RSEED,RANG(W_RANG),CNUG(W_CNUG)                             

C        COMMON /SIM/   RF(65,65,6)                                      
         COMMON /SIM/ RF(W_RFX,W_RFY,W_RFZ)                                        

        COMMON /GRID/  NROW,NCOL,XDIM,YDIM,YMAX,XMIN                    

        DOUBLE PRECISION RSEED                                          
        DUM        =   CNUG(K)                                          
        STD        =   SQRT(DUM)                                        
        IF(STD.EQ.0.0) GO TO 100                                        
        DO 90   I  =   1,NROW                                           
        DO 90   J  =   1,NCOL                                           
        W          =   (DRAND(RSEED) - 0.50) * STD * 5.0                
        RF(I,J,K)  =   RF(I,J,K) + W                                    
90      CONTINUE                                                        
100     CONTINUE                                                        
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        SUBROUTINE PROPT                                                

        COMMON /OPT/ NSTOP,KTEST,IKRIG,ITRANS                           

C...                                                                    
C.......SUBROUTINE TO PRINT PROGRAM OPTIONS.                            
C...                                                                    
        WRITE(6,50)                                                     
        IF(NSTOP.EQ.0) WRITE(6,100)                                     
        IF(NSTOP.NE.0) WRITE(6,150)                                     
        IF(KTEST.EQ.0) WRITE(6,200)                                     
        IF(KTEST.NE.0) WRITE(6,250)                                     
        IF(IKRIG.EQ.0) WRITE(6,300)                                     
        IF(IKRIG.NE.0) WRITE(6,350)                                     
        IF(ITRANS.EQ.0) WRITE(6,400)                                    
        IF(ITRANS.NE.0) WRITE(6,450)                                    
50      FORMAT(1H1,T30,'PROGRAM OPTIONS IN EFFECT:',//)                 
100     FORMAT(T30,'NON-CONDITIONAL SIMULATION ONLY',/)                 
150     FORMAT(T30,'SIMULATION WAS CONDITIONED',/)                      
200     FORMAT(T30,'NO TEST LOCATIONS DESIRED',/)                       
250     FORMAT(T30,'TEST LOCATIONS WERE SPECIFIED',/)                   
300     FORMAT(T30,'ALL VARIABLES WERE FULLY SAMPLED',/)                
350     FORMAT(T30,'SOME VARIABLES WERE UNDER-SAMPLED',/)               
400     FORMAT(T30,'LINEAR TRANSFORMATION EFFECTED',/)                  
450     FORMAT(T30,'HERMITIAN TRANSFORMATION EFFECTED',/)               
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        SUBROUTINE PRT1(I,J,K,C,BSEED,RMEAN,IVAR)                       
        INCLUDE 'SIZES.INC'

        COMMON /GRID/ NROW,NCOL,XDIM,YDIM,YMAX,XMIN                     

C        COMMON /MODEL/ RSEED,RANG(5),CNUG(5)                            
         COMMON /MODEL/RSEED,RANG(W_RANG),CNUG(W_CNUG)                             

        DOUBLE PRECISION RSEED,BSEED                                    
C...                                                                    
C.......SUBROUTINE TO PRINT INITIAL PROGRAM RESULTS AND CONSTANTS.      
C...                                                                    
C.......1.  GRID DEFINITION.                                            
C...                                                                    
        WRITE(6,50) IVAR                                                
50      FORMAT(1H1,T30,'*****  CO-SIMULATION PROGRAM *****',//,         
     1  T30,'RESULTS FOR VARIABLE ',I10,//)                             
        WRITE(6,100) NROW,NCOL,XDIM,YDIM,YMAX,XMIN                      
100     FORMAT(1H0,T30,'CO-SIMULATION GRID DIMENSIONS',//,              
     1  T20,'NUMBER OF ROWS       =  ',T50,I10,/,                       
     2  T20,'NUMBER OF COLUMNS    =  ',T50,I10,/,                       
     3  T20,'INCREMENT IN X       =  ',T50,F10.3,/,                     
     4  T20,'INCREMENT IN Y       =  ',T50,F10.3,/,                     
     5  T20,'MAXIMUM Y COORDINATE =  ',T50,F10.3,/,                     
     6  T20,'MINIMUM X COORDINATE =  ',T50,F10.3,//)                    
C...                                                                    
C.......2.  MODEL PARAMETERS                                            
C...                                                                    
        WRITE(6,200) BSEED,RSEED,RMEAN,RANG(IVAR),CNUG(IVAR)            
200     FORMAT(//,T30,'MODEL PARAMETERS',//,                            
     1  T20,'BEGINNING RANDOM SEED      = ',T50,D20.11,/,               
     2  T20,'FINAL RANDOM NUMBER SEED   = ',T50,D20.11,/,               
     3  T20,'MEAN OF RANDOM NUMBERS     = ',T50,F20.3,/,                
     4  T20,'RANGE OF SPATIAL STRUCTURE = ',T50,F20.3,/,                
     5  T20,'VARIANCE OF SPATIAL NOISE  = ',T50,F20.3,//)               
C...                                                                    
C.......3.  CONSTANTS COMPUTED BY PROGRAM                               
C...                                                                    
        WRITE(6,300) I,J,K,C                                            
300     FORMAT(//,T30,'INITIAL CONSTANTS',//,                           
     1  T20,'NGMAX      =  ',T50,I10,/,                                 
     2  T20,'NR         =  ',T50,I10,/,                                 
     3  T20,'KD         =  ',T50,I10,/,                                 
     4  T20,'C          =  ',T50,F10.3,//)                              
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        SUBROUTINE PRT2(IJK)                                            
        INCLUDE 'SIZES.INC'

        DIMENSION BUF(100)                                              

C        COMMON /SIM/ RF(65,65,6)                                        
         COMMON /SIM/ RF(W_RFX,W_RFY,W_RFZ)                                        

        COMMON /GRID/ NROW,NCOL,XDIM,YDIM,YMAX,XMIN                     

C        COMMON /HIST/ BIN(10)                                           
         COMMON /HIST/ BIN(W_BIN)

        INTEGER BUF,BIN                                                 
C...                                                                    
C.......THIS SUBROUTINE PRINTS A PICTURE OF THE SIMULATION RESULTS.     
C...                                                                    
        DO 5   I   =  1,10                                              
        BIN(I)     =  0                                                 
5       CONTINUE                                                        
        N          =  NCOL                                              
        IF(NCOL.GT.70) N = NCOL / 2                                     
C...                                                                    
C.......FIND THE MAXIMUM AND MINIMUM VALUES OF THE SIMULATED RF.        
C...                                                                    
        RFMIN      =  73057.00                                          
        RFMAX      = -73057.00                                          
        DO 10  I   =  1,NROW                                            
        DO 10  J   =  1,NCOL                                            
        B          =  RF(I,J,IJK)                                       
        IF(B.LT.RFMIN) RFMIN = B                                        
        IF(B.GT.RFMAX) RFMAX = B                                        
10      CONTINUE                                                        
        DIFF       =  RFMAX - RFMIN                                     
        RFINC      =  DIFF / 10.0                                       
        DO 20  I   =  1,NROW                                            
        DO 20  J   =  1,NCOL                                            
        DO 20  K   =  1,10                                              
        XOLD       =  FLOAT(K - 1)*RFINC + RFMIN                        
        XNEW       =  FLOAT(K)    *RFINC + RFMIN                        
        B          =  RF(I,J,IJK)                                       
        IF(B.GE.XOLD.AND.B.LT.XNEW) BIN(K) = BIN(K) + 1                 
20      CONTINUE                                                        
        WRITE(6,30)                                                     
30      FORMAT(1H1,T30,'HISTOGRAM OF SIMULATION',//)                    
        CALL HGRAM(RFMIN,RFMAX)                                         
        WRITE(6,400)                                                    
        IF(N.NE.NCOL) GO TO 200                                         
        DO 100 I   =  1,NROW                                            
        DO  90 J   =  1,NCOL                                            
        L          =  0                                                 
        IF(RF(I,J,IJK).EQ.RFMIN) GO TO 60                               
        DO  50 K   =  1,10                                              
        L          =  K - 1                                             
        VAL2       =  RFMIN + RFINC * FLOAT(K)                          
        VAL1       =  RFMIN + RFINC * FLOAT(L)                          
        B          =  RF(I,J,IJK)                                       
        IF(B.GT.VAL1.AND.B.LE.VAL2) GO TO 60                            
50      CONTINUE                                                        
60      CONTINUE                                                        
        BUF(J)     =  L                                                 
90      CONTINUE                                                        
        WRITE(6,500) (BUF(JK),JK = 1,N)                                 
100     CONTINUE                                                        
        GO TO 300                                                       
200     CONTINUE                                                        
        DO 290  I  =  1,NROW,2                                          
        J1         =  0                                                 
        DO 280  J  =  1,NCOL,2                                          
        J1         =  J1 + 1                                            
        L          =  0                                                 
        IF(RF(I,J,IJK).EQ.RFMIN) GO TO 260                              
        DO 210  K  =  1,10                                              
        L          =  K - 1                                             
        VAL2       =  RFMIN + RFINC * FLOAT(K)                          
        VAL1       =  RFMIN + RFINC * FLOAT(L)                          
        B          =  RF(I,J,IJK)                                       
        IF(B.GT.VAL1.AND.B.LE.VAL2) GO TO 260                           
210     CONTINUE                                                        
260     CONTINUE                                                        
        BUF(J1)    =  L                                                 
280     CONTINUE                                                        
        WRITE(6,500) (BUF(JK),JK = 1,N)                                 
290     CONTINUE                                                        
300     CONTINUE                                                        
400     FORMAT(1H1,T30,'SIMULATION RESULTS',//,                         
     1  T20,'IF NCOL.GT.70, THE FOLLOWING',/,                           
     2  T20,'IS EVERY OTHER COLUMN BY EVERY',/,                         
     3  T20,'OTHER ROW.',//)                                            
500     FORMAT(T3,75I1)                                                 
C...                                                                    
C.......COMPUTE LEGEND                                                  
C...                                                                    
        WRITE(6,700)                                                    
        DO 600  I  =  1,10                                              
        J          =  I - 1                                             
        DUM1       =  RFMIN + RFINC * FLOAT(J)                          
        DUM2       =  RFMIN + RFINC * FLOAT(I)                          
        WRITE(6,800) J,DUM1,DUM2                                        
600     CONTINUE                                                        
700     FORMAT(1H1,T30,'LEGEND',//,T22,'MAP VALUE',/,                   
     1  T37,'LOW RANGE',T51,'HIGH RANGE',/)                             
800     FORMAT(T20,I10,T35,E10.4,T50,E10.4)                             
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        SUBROUTINE PRT3(KOUNT)                                          
        INCLUDE 'SIZES.INC'

C        COMMON /VAR/  CO(5),C(5),RANGE(5),ANIS(5),RATIO(5)              
         COMMON /VAR/ CO(W_CO),C(W_C),RANGE(W_RNG),
     1                   ANIS(W_AN),RATIO(W_RAT)

C        COMMON /DAT2/ X(100),Y(100),DAT(100,5)                          
         COMMON /DAT2/ X(W_X),Y(W_Y),DAT(W_DAT1,W_DAT2)                          

C        COMMON /CVAR/ CCO(10),CC(10),CRANGE(10),CANIS(10),CRATIO(10)    
         COMMON /CVAR/ CCO(W_CCO),CC(W_CC),CRANGE(W_CRAN),
     1                   CANIS(W_CAN),CRATIO(W_CRAT)    

        COMMON /PARM/ MVAR                                              
C...                                                                    
C.......THIS SUBROUTINE PRINTS INITIAL CONDITIONING INFORMATION.        
C...                                                                    
        WRITE(6,100)                                                    
        DO 10   I  =  1,MVAR                                            
        WRITE(6,200) I,CO(I),C(I),RANGE(I),ANIS(I),RATIO(I)             
10      CONTINUE                                                        
        IF(MVAR.EQ.1) GO TO 25                                          
        N          =  (MVAR * (MVAR - 1)) / 2                           
        WRITE(6,300)                                                    
        DO 20   I  =  1,N                                               
        WRITE(6,200) I,CCO(I),CC(I),CRANGE(I),CANIS(I),CRATIO(I)        
20      CONTINUE                                                        
25      CONTINUE                                                        
        WRITE(6,400)                                                    
        DO 30   I  =  1,KOUNT                                           
        WRITE(6,500) X(I),Y(I),(DAT(I,JK),JK = 1,MVAR)                  
30      CONTINUE                                                        
100     FORMAT(1H1,T30,'**** CONDITIONING RESULTS ****',//,             
     1  T30,'VARIOGRAM PARAMETERS',//,                                  
     2  T3,'VARIABLE',T15,'NUGGET',T27,'SILL',T36,'RANGE',T47,          
     3  'ANIS',T56,'RATIO',//)                                          
200     FORMAT(I10,5F10.3)                                              
300     FORMAT(//,T30,'CROSS-CORRELATION PARAMETERS',//,                
     1  T7,'PAIR',T15,'NUGGET',T27,'SILL',T36,'RANGE',T47,'ANIS',       
     2  T56,'RATIO',//)                                                 
400     FORMAT(//,T30,'CONDITIONING DATA',//,                           
     1  T7,'EAST',T16,'NORTH',T27,'VAR1',T37,'VAR2',T47,'VAR3',         
     2  T57,'VAR4',T67,'VAR5',/)                                        
500     FORMAT(7F10.3)                                                  
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        SUBROUTINE PTEST(K)                                             
        INCLUDE 'SIZES.INC'

C        COMMON /TEST/ TX(20),TY(20)                                     
         COMMON /TEST/ TX(W_TX),TY(W_TY)                                     

        COMMON /PARM/ MVAR                                              

C        COMMON /SIM/  RF(65,65,6)                                       
         COMMON /SIM/ RF(W_RFX,W_RFY,W_RFZ)                                        

        COMMON /GRID/ NROW,NCOL,XDIM,YDIM,YMAX,XMIN                     

C...                                                                    
C.......SUBROUTINE PTEST PRINTS THE VALUES WITHIN THE SIMULATED         
C.......MODEL CLOSEST TO LOCATIONS: (TX,TY).                            
C...                                                                    
        WRITE(6,100)                                                    
        YBEGIN     =  YMAX   +  0.5 * YDIM                              
        XBEGIN     =  XMIN   -  0.5 * XDIM                              
        DO 10   I  =  1,K                                               
        IROW       =  (YBEGIN - TY(I)) / YDIM                           
        JCOL       =  (TX(I) - XBEGIN) / XDIM                           
        IF(IROW.LE.0) IROW = 1                                          
        IF(IROW.GT.NROW) IROW = NROW                                    
        IF(JCOL.LE.0) JCOL = 1                                          
        IF(JCOL.GT.NCOL) JCOL = NCOL                                    
        WRITE(6,200) IROW,JCOL,TX(I),TY(I),(RF(IROW,JCOL,JK),           
     1               JK = 1,MVAR)                                       
10      CONTINUE                                                        
100     FORMAT(1H1,T30,'SIMULATED VALUES AT TEST LOCATIONS',//,         
     1  T4,'ROW',T9,'COL',T20,'X',T30,'Y',T37,'VAR1',T47,'VAR2',        
     2  T57,'VAR3',T67,'VAR4',T77,'VAR5',/)                             
200     FORMAT(T3,2I4,7F10.3)                                           
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        SUBROUTINE SCALG(A,B,X)                                         
        COMMON /PARM/ MVAR                                              
        DIMENSION A(5,5),B(5,5),X(5,5),TEMP(5,10)                       
C...                                                                    
C.......THIS SUBROUTINE PERFORMS THE NORMALIZATION BY THE               
C.......PIVOT TERM FOR GAUSS ELIMINATION MATRIX INVERSION.              
C...                                                                    
        MVAR2      =  MVAR * 2                                          
        DO 10  I   =  1,MVAR                                            
        DO 10  J   =  1,MVAR                                            
        TEMP(I,J)  =  A(I,J)                                            
10      CONTINUE                                                        
        DO 20  I   =  1,MVAR                                            
        DO 20  J   =  1,MVAR                                            
        K          =  MVAR + J                                          
        TEMP(I,K)  =  B(I,J)                                            
20      CONTINUE                                                        
        DO 30  I   =  1,MVAR                                            
        IP         =  I + 1                                             
        DO 30  K   =  1,MVAR                                            
        IF(I-K) 26,30,26                                                
26      IF(TEMP(I,I).EQ.0.0)  GO TO 30                                  
        F          =  (-TEMP(K,I)) / TEMP(I,I)                          
        DO 27  L   =  IP,MVAR2                                          
27      TEMP(K,L)  =  TEMP(K,L) + F*TEMP(I,L)                           
30      CONTINUE                                                        
        DO 40  I   =  1,MVAR                                            
        DO 40  J   =  1,MVAR                                            
        K          =  MVAR + J                                          
        IF(TEMP(I,I).EQ.0.0)  GO TO 40                                  
        X(I,J)     =  TEMP(I,K) / TEMP(I,I)                             
40      CONTINUE                                                        
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        FUNCTION TRACE(A)                                               
        COMMON /PARM/ MVAR                                              
        REAL A(5,5)                                                     
C...                                                                    
C.......THIS FUNCTION COMPUTES THE TRACE OF SQUARE MATRIX, A.           
C...                                                                    
        TRACE      =  0.0                                               
        DO 10 I    =  1,MVAR                                            
        TRACE      =  TRACE + A(I,I)                                    
10      CONTINUE                                                        
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        SUBROUTINE TRANS(XMEAN,VAR,K)                                   
        INCLUDE 'SIZES.INC'

C        COMMON /SIM/ RF(65,65,6)                                        
         COMMON /SIM/ RF(W_RFX,W_RFY,W_RFZ)                                        

C        COMMON /LTRAN/ LVAR(5),LMEAN(5)                                 
         COMMON /LTRAN/LVAR(W_LVAR), LMEAN(W_LMEAN)

        COMMON /GRID/ NROW,NCOL,XDIM,YDIM,YMAX,XMIN                     

C        COMMON /MODEL/ RSEED,RANG(5),CNUG(5)                            
         COMMON /MODEL/RSEED,RANG(W_RANG),CNUG(W_CNUG)                             

C        COMMON /HERM/ COEF(10,5)                                        
         COMMON /HERM/ COEF(W_COEF1,W_COEF2)                                        

        COMMON /OPT/ NSTOP,KTEST,IKRIG,ITRANS                           

        REAL LVAR,LMEAN,C(12)                                           
        DOUBLE PRECISION RSEED                                          
C...                                                                    
C.......THIS SUBROUTINE PROVIDES A LINEAR OR HERMITIAN                  
C.......TRANSFORMATION FROM A DISTRIBUTION (0,1).  FOR THE LINEAR       
C.......TRANSFORMATION, THE PROCEDURE USED IS:                          
C.......                                                                
C.......                   T = (I - M1) * S1/S2 + M2                    
C.......     WHERE,                                                     
C.......                   T = TRANSFORMED VALUE                        
C.......                   I = SIM. VALUE IN ORIGINAL DISTRIBUTION      
C.......                  M1 = MEAN OF ORIGINAL DISTRIBUTION            
C.......                  M2 = MEAN OF TRANSFORMED DISTRIBUTION         
C.......                  S2 = STANDARD DEVIATION OF ORIGINAL DIST.     
C.......                  S1 = STANDARD DEVIATION OF TRANSFORMED DIST.  
C...                                                                    
        DO 5    I  =  1,12                                              
        C(I)       =  0.0                                               
5       CONTINUE                                                        
        IF(ITRANS.EQ.1) GO TO 200                                       
        S1         =  SQRT(LVAR(K))                                     
        S2         =  SQRT(VAR)                                         
        DO 100  I  =  1,NROW                                            
        DO 100  J  =  1,NCOL                                            
        RF(I,J,K)  =  (RF(I,J,K) - XMEAN) * S1/S2 + LMEAN(K)            
100     CONTINUE                                                        
C...                                                                    
C.......ADD A NUGGET EFFECT TO THE MODEL IF DESIRED.                    
C...                                                                    
        CALL NUGGET(K)                                                  
        RETURN                                                          
200     CONTINUE                                                        
C...                                                                    
C.......HERMITIAN TRANSFORMATION.                                       
C.......CORRECT THE MODEL FOR DEFICIENCIES THEN APPLY TRANSFORMATION    
C...                                                                    
        S2         =  SQRT(VAR)                                         
        DO 210  I  =  1,NROW                                            
        DO 210  J  =  1,NCOL                                            
        X          =  (RF(I,J,K) - XMEAN) * 1.0 / S2                    
        DO 205  M  =  10,1,-1                                           
        C(M)       =  COEF(M,K) + X * C(M+1) - M*C(M+2)                 
205     CONTINUE                                                        
        RF(I,J,K)  =  C(1)                                              
        DO 210  N  =  1,12                                              
        C(N)       =  0.0                                               
210     CONTINUE                                                        
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        SUBROUTINE UNSAM(K)                                             
        INCLUDE 'SIZES.INC'

C        COMMON /FORM/ A(100,100)                                        
         COMMON /FORM/ A(W_AX,W_AY)                                        
        
C        COMMON /DAT2/ X(100),Y(100),DAT(100,5)                          
         COMMON /DAT2/ X(W_X),Y(W_Y),DAT(W_DAT1,W_DAT2)                          

        COMMON /PARM/ MVAR                                              
C...                                                                    
C.......THIS SUBROUTINE MODIFIES THE MATRIX, A, TO ACCOUNT              
C.......FOR UNDERSAMPLED RANDOM FUNCTIONS.  A ZERO DATA                 
C.......VALUE IS INDICATIVE OF UNDERSAMPLING.                           
C...                                                                    
        L          =  K + 1                                             
        LVAR       =  L * MVAR                                          
        DO 100  I  =  1,K                                               
        DO 90   J  =  1,MVAR                                            
        IF(DAT(I,J).NE.0.0) GO TO 90                                    
        NI         =  (I - 1) * MVAR + J                                
        DO 80   M  =  1,LVAR                                            
        A(NI,M)    =  0.0                                               
        A(M,NI)    =  0.0                                               
80      CONTINUE                                                        
90      CONTINUE                                                        
100     CONTINUE                                                        
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        SUBROUTINE UNVEC(XX,K)                                          
        INCLUDE 'SIZES.INC'

C        COMMON /DAT2/ X(100),Y(100),DAT(100,5)                          
         COMMON /DAT2/ X(W_X),Y(W_Y),DAT(W_DAT1,W_DAT2)                          

        COMMON /PARM/ MVAR                                              

        DIMENSION XX(100,5)                                             
C...                                                                    
C.......THIS SUBROUTINE MODIFIES THE MEASUREMENT VECTOR TO              
C.......ACCOUNT FOR UNDERSAMPLED RANDOM FUNCTIONS.  A ZERO              
C.......DATA VALUE IS INDICATIVE OF UNDERSAMPLING.                      
C...                                                                    
        DO 100  I  =  1,K                                               
        DO 90   J  =  1,MVAR                                            
        IF(DAT(I,J).NE.0.0) GO TO 90                                    
        NI         =  (I - 1) * MVAR + J                                
        DO 80   M  =  1,MVAR                                            
        XX(NI,M)    =  0.0                                              
80      CONTINUE                                                        
90      CONTINUE                                                        
100     CONTINUE                                                        
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        SUBROUTINE VARG(XMEAN,VAR,IJK)                                  
        INCLUDE 'SIZES.INC'

C        COMMON /SIM/ RF(65,65,6)                                        
         COMMON /SIM/ RF(W_RFX,W_RFY,W_RFZ)                                        

        COMMON /GRID/ NROW,NCOL,XDIM,YDIM,YMAX,XMIN                     

        DIMENSION GAMMA(20)                                             
C...                                                                    
C.......THIS SUBROUTINE COMPUTES THE SEMI-VARIOGRAM OF THE              
C.......SIMULATED RANDOM FUNCTION, RF.  THE REGULAR GRID                
C.......SPACING IS TAKEN ADVANTAGE OF IN THE EAST-WEST DIRECTION.       
C...                                                                    
C.......1.  COMPUTE THE MEAN AND VARIANCE OF THE SIMULATION.            
C...                                                                    
        DO 5  I    =  1,20                                              
        GAMMA(I)   =  0.0                                               
5       CONTINUE                                                        
        VAR        =  0.0                                               
        XMEAN      =  0.0                                               
        DO 10 I    =  1,NROW                                            
        DO 10 J    =  1,NCOL                                            
        XMEAN      =  XMEAN + RF(I,J,IJK)                               
10      CONTINUE                                                        
        NUM        =  NROW * NCOL                                       
        XMEAN      =  XMEAN / FLOAT(NUM)                                
        DO 20 I    =  1,NROW                                            
        DO 20 J    =  1,NCOL                                            
        VAR        =  VAR + (RF(I,J,IJK) - XMEAN)**2                    
20      CONTINUE                                                        
        VAR        =  VAR / FLOAT(NUM - 1)                              
        NCOL1      =  NCOL / 20                                         
        DO 1000  K =  1,20                                              
        KOUNT      =  0                                                 
        DUM1       =  0.0                                               
        L          =  K * NCOL1                                         
        I1         =  NCOL - L + 2                                      
        DO 100 I   =  1,NROW                                            
        DO 100 J   =  1,I1                                              
        I4         =  J + L                                             
        IF(I4.GT.NCOL) GO TO 100                                        
        KOUNT      =  KOUNT + 1                                         
        C          =  RF(I,I4,IJK)                                      
        D          =  RF(I,J,IJK)                                       
        DUM1       =  DUM1 + (C - D)**2                                 
        IF(KOUNT.GT.5000) GO TO 200                                     
100     CONTINUE                                                        
200     CONTINUE                                                        
        N2         =  2 * KOUNT                                         
        IF(N2.EQ.0) GO TO 1000                                          
        GAMMA(K)   =  DUM1 / FLOAT(N2)                                  
1000    CONTINUE                                                        
        WRITE(6,2000)                                                   
        DO 1500 I = 1,20                                                
        WRITE(6,3000) I,GAMMA(I)                                        
1500    CONTINUE                                                        
2000    FORMAT(1H1,T30,'VARIOGRAM RESULTS',//,                          
     1  T25,'CLASS',T35,'GAMMA',/)                                      
3000    FORMAT(T20,I10,T30,F10.3)                                       
        WRITE(6,4000) XMEAN, VAR                                        
4000    FORMAT(/,T30,'MEAN AND VARIANCE OF THE SIMULATION',/,           
     1  T20,'MEAN = ',E15.3,T50,'VARIANCE = ',E15.3,/)                  
        RETURN                                                          
        END                                                             
C                                                                       
C                                                                       
        SUBROUTINE VECT                                                 
        INCLUDE 'SIZES.INC'

C        COMMON /DVEC/ ICOS(3,15)                                        
         COMMON /DVEC/ ICOS(W_ICOS1,W_ICOS2)         

        DIMENSION R(3,3),TEMP(3,3),TEMP1(3,3)                           
C...                                                                    
C.......THIS SUBROUTINE DEFINES THE 15 VECTORS THROUGH ROTATION BY R.   
C...                                                                    
        REAL K,ICOS                                                     
        K          =  0.618033989                                       
        ICOS(1,1)  =  K                                                 
        ICOS(1,2)  =  1.0                                               
        ICOS(1,3)  =  1.0 + K                                           
        ICOS(2,1)  =  1.0                                               
        ICOS(2,2)  =  -1.0 - K                                          
        ICOS(2,3)  =  K                                                 
        ICOS(3,1)  =  K + 1.0                                           
        ICOS(3,2)  =  K                                                 
        ICOS(3,3)  =  -1.0                                              
        R(1,1)     =  0.50                                              
        R(1,2)     =  -(K + 1.0) / 2.0                                  
        R(1,3)     =  K / 2.0                                           
        R(2,1)     =  (K + 1.0) / 2.0                                   
        R(2,2)     =  K / 2.0                                           
        R(2,3)     =  -0.50                                             
        R(3,1)     =  K / 2.0                                           
        R(3,2)     =  0.50                                              
        R(3,3)     =  (K + 1.0) / 2.0                                   
C...                                                                    
C.......COMPUTE REMAINING 12 VECTORS                                    
C...                                                                    
        DO 100  M  =  1,4                                               
        DO 10   I  =  1,3                                               
        DO 10   J  =  1,3                                               
        N          =  (M - 1) * 3  +  J                                 
        TEMP(I,J)  =  ICOS(I,N)                                         
10      CONTINUE                                                        
        CALL MATML1(TEMP,R,TEMP1,3,3,3)                                 
C...                                                                    
C.......TEMP1 CONTAINS NEW ROTATED VECTORS; THIS IS PLACED INTO ICOS.   
C...                                                                    
        DO 20  I   =  1,3                                               
        DO 20  J   =  1,3                                               
        N          =  M * 3 + J                                         
        ICOS(I,N)  =  TEMP1(I,J)                                        
20      CONTINUE                                                        
100     CONTINUE                                                        
        RETURN                                                          
        END                                                             

C-----------------------------------------------------------------------
C  SUBROUTINE WRITE_XYZ
C  Purpose:  To write the conditional simulation results contained in array
C           RF to a plain text XYZ format file for use with other programs.
C           1/15/92 GJR.

        SUBROUTINE WRITE_XYZ 
        INCLUDE 'SIZES.INC'

C        COMMON /SIM/ RF(65,65,6)
         COMMON /SIM/ RF(W_RFX,W_RFY,W_RFZ)                                        

        COMMON /GRID/ NROW,NCOL,XDIM,YDIM,YMAX,XMIN
        COMMON /PARM/ MVAR

        INTEGER*2       IUNIT

        OPEN(7, FILE = 'COND.OUT')
        IUNIT = 7

        YBEGIN     =  YMAX + 0.5 * YDIM
        XBEGIN     =  XMIN - 0.5 * XDIM

        DO 100 K = 1, MVAR

        DO 1 I = 1, NROW
                Y = YBEGIN - FLOAT(I) * YDIM

            DO 2 J = 1, NCOL
                X = XBEGIN + FLOAT(J) * XDIM
                WRITE(IUNIT,*)X, Y, RF(I,J,K)
2           CONTINUE
1       CONTINUE


100     CONTINUE
        CLOSE (IUNIT)

        RETURN
        END
C************************************************************************
      SUBROUTINE HEADER
C
C========================================================================
C
C------- OUTPUT HEADER TO SCREEN.
      CALL CLRSCR
      CALL RVRSVD
      CALL BOX (1,7,1,80)

      CALL XYTAB (2,33)
      WRITE (*,10)
10    FORMAT ('C O S I M - 3 8 6',\)

      CALL XYTAB (3,18)
      WRITE(*,21)
21    FORMAT('Unconditional and Conditional Geostatistical',\)

      CALL XYTAB (4,33)
      WRITE(*,25)
25    FORMAT('Simulation Program',/)

      CALL XYTAB (5,20)
      WRITE (*,20)
20    FORMAT ('INTERA Inc., Albuquerque, New Mexico USA ',\)

      CALL XYTAB (6,35)
      WRITE (*,30)
30    FORMAT ('Version 2.0',\)
      CALL XYTAB (8,1)
      CALL NORMVD
C
      RETURN
      END
C***********************************************************************
      SUBROUTINE CLRSCR
C      
      CHARACTER*1 ESC, BACKSP, NUL
C
      ESC = CHAR(27)
      BACKSP = CHAR(8)
      NUL = CHAR(0)
C      WRITE (*,10) NUL, BACKSP, ESC
C10    FORMAT (3A1,'[2J',\)
      WRITE (*,10) NUL,ESC
10    FORMAT (2A1,'[2J',\)
C
      RETURN
      END
C**********************************************************************
      SUBROUTINE BLNKRC (I1,I2,J1)
C
      CHARACTER*1 ESC, NUL
C
C------ ERASE A SECTION OF THE SCREEN
C
      ESC = CHAR(27)
      NUL = CHAR(0)
      DO 20 I=I1,I2
        CALL XYTAB (I,J1)
        WRITE (*,10) NUL, ESC
10      FORMAT (2A1,'[K',\)
20    CONTINUE
C
C------ RESTORE CURSOR POSITION FOR SUBSEQUENT PRINTING
C
      CALL XYTAB (I1,J1)
C
      RETURN
      END
C**********************************************************************
      SUBROUTINE BLNKC (I1,J1,J2)
C
C------- ERASE REQUESTED COLUMNS OF A LINE
C
      DO 10 J=J1,J2-1
        CALL XYTAB (I1,J)
        WRITE (*,20)
20      FORMAT (' ',\)
10    CONTINUE
C
C------- RESTORE CURSOR POSITION FOR SUBSEQUENT PRINTING
C
      CALL XYTAB (I1,J1)
C
      RETURN
      END
C**********************************************************************
      SUBROUTINE NORMVD
C
      CHARACTER*1 ESC, BACKSP, NUL
C
      ESC = CHAR(27)
      BACKSP = CHAR(8)
      NUL = CHAR(0)
      WRITE (*,10) NUL, BACKSP, ESC
10    FORMAT (3A1,'[0m',\)
C
      RETURN
      END
C**********************************************************************
      SUBROUTINE BLNKVD
C
      CHARACTER*1 ESC, BACKSP, NUL
C
      ESC = CHAR(27)
      BACKSP = CHAR(8)
      NUL = CHAR(0)
      WRITE (*,10) NUL, BACKSP, ESC
10    FORMAT (3A1,'[5m',\)
C     
      RETURN
      END
C**********************************************************************
      SUBROUTINE BOLDVD
C
      CHARACTER*1 ESC, BACKSP, NUL
C
      ESC = CHAR(27)
      BACKSP = CHAR(8)
      NUL = CHAR(0)
      WRITE (*,10) NUL, BCKSP, ESC
10    FORMAT (3A1,'[1m',\)
C
      RETURN
      END      
C**********************************************************************
      SUBROUTINE RVRSVD
C
      CHARACTER*1 ESC, BACKSP, NUL
C
      ESC = CHAR(27)
      BACKSP = CHAR(8)
      NUL = CHAR(0)
      WRITE (*,10) NUL, BACKSP, ESC
10    FORMAT (3A1,'[7m',\)      
C
      RETURN
      END
C***********************************************************************
      SUBROUTINE UNDRVD
C
      CHARACTER*1 ESC, BACKSP, NUL
C
      ESC = CHAR(27)
      BACKSP = CHAR(8)
      NUL = CHAR(0)
      WRITE (*,10) NUL, BACKSP, ESC
10    FORMAT (3A1,'[4m',\)
C 
      RETURN
      END      
C***********************************************************************
      SUBROUTINE NOWRAP
C
      CHARACTER*1 ESC, BACKSP, NUL
C
C------- DEACTIVATE LINE WRAP
C 
      ESC=CHAR(27)
      BACKSP = CHAR(8)
      NUL=CHAR(0)
      WRITE (*,10) NUL, BACKSP, ESC
10    FORMAT (3A1,'[?7l',\)
C
      RETURN
      END         
C***********************************************************************
      SUBROUTINE CPSAVE
C
      CHARACTER*1 ESC, BACKSP, NUL
C
C------- SAVE CURSOR POSITION
C
      ESC = CHAR(27)
      BACKSP = CHAR(8)
      NUL = CHAR(0)
      WRITE (*,10) NUL, BACKSP, ESC
10    FORMAT (3A1,'[s',\)
C
      RETURN
      END
C***********************************************************************
      SUBROUTINE CPRSTR
C
      CHARACTER*1 ESC, NUL
C
C------- RESTORE CURSOR POSITION
C
      ESC = CHAR(27)
      NUL = CHAR(0)
      WRITE (*,10) NUL, ESC
10    FORMAT (2A1,'[u',\)
C  
      RETURN
      END            
C***********************************************************************
      SUBROUTINE XYTAB (NROW,NCOL)
C
      CHARACTER*1 ESC, BACKSP, NUL
      CHARACTER FTAB*15
C
C------- LOCATE CURSOR TO REQUESTED ROW AND COLUMN
C
      ESC = CHAR(27)
      BACKSP = CHAR(8)
      NUL = CHAR(0)
      IF (NROW.LT.10.AND.NCOL.LT.10) THEN
        WRITE (FTAB,10) NUL,BACKSP,ESC,NROW,NCOL
10      FORMAT (3A1,'[',I1,';',I1,'f')
        WRITE (*,20) FTAB
20      FORMAT (A8,\)
      ELSEIF (NROW.LT.10.AND.NCOL.GE.10) THEN
        WRITE (FTAB,30) NUL,BACKSP,ESC,NROW,NCOL
30      FORMAT (3A1,'[',I1,';',I2,'f')
        WRITE (*,40) FTAB
40      FORMAT (A9,\)
      ELSEIF (NROW.GE.10.AND.NCOL.LT.10) THEN
        WRITE (FTAB,50) NUL,BACKSP,ESC,NROW,NCOL
50      FORMAT (3A1,'[',I2,';',I1,'f')
        WRITE (*,60) FTAB
60      FORMAT (A10,\)
      ELSE 
        WRITE (FTAB,70) NUL,BACKSP,ESC,NROW,NCOL
70      FORMAT (3A1,'[',I2,';',I2,'f')
        WRITE (*,80) FTAB
80      FORMAT (A10,\)
      ENDIF
C
      RETURN
      END
C***********************************************************************
      SUBROUTINE XTAB (NCOL)
C
      CHARACTER*1 ESC, BACKSP, NUL, DIR
      CHARACTER FTAB*7
C
      ESC = CHAR(27)
      BACKSP = CHAR(8)
      NUL = CHAR(0)
      IF (NCOL.LT.0) THEN
        DIR = 'D'
      ELSE
        DIR = 'C'
      ENDIF
      NC = ABS(NCOL)
      IF (NC.LT.10) THEN
        WRITE (FTAB,10) NUL, BACKSP, ESC, NC, DIR
10      FORMAT (3A1,'[',I1,A1)
        WRITE (*,20) FTAB
20      FORMAT (A6,\)
      ELSE
        WRITE (FTAB,30) NUL, BACKSP, ESC, NC, DIR
30      FORMAT (3A1,'[',I2,A1)
        WRITE (*,40) FTAB
40      FORMAT (A7,\)
      ENDIF
C
      RETURN
      END      
C***********************************************************************
      SUBROUTINE YTAB (NROW)
C
      CHARACTER*1 ESC, BACKSP, NUL, DIR
      CHARACTER FTAB*7
C
      ESC = CHAR(27)
      BACKSP = CHAR(8)
      NUL = CHAR(0)
      IF (NROW.LT.0) THEN
        DIR = 'A'
      ELSE
        DIR = 'B'
      ENDIF
      NR = ABS (NROW)
      IF (NR.LT.10) THEN
        WRITE (FTAB,10) NUL, BACKSP, ESC, NR, DIR
10      FORMAT (3A1,'[',I1,A1)
        WRITE (*,20) FTAB
20      FORMAT (A6,\)
      ELSE
        WRITE (FTAB,30) NUL, BACKSP, ESC, NR, DIR
30      FORMAT (3A1,'[',I2,A1)
        WRITE (*,40) FTAB
40      FORMAT (A7,\)
      ENDIF
C
      RETURN
      END
C***********************************************************************
      SUBROUTINE BOX (I1,I2,J1,J2)
C
      CHARACTER   P1*80
      CHARACTER*1 DHBAR, DVBAR, ULC, URC, LLC, LRC, BLANK, P(80)
C
      EQUIVALENCE (P(1),P1)
C
      DHBAR=CHAR(205)
      DVBAR=CHAR(186)
      ULC=CHAR(201)
      URC=CHAR(187)
      LLC=CHAR(200)
      LRC=CHAR(188)
      BLANK=CHAR(32)
C
      CALL XYTAB (I1,1)
      DO 10 J=1,80
        P(J)=BLANK
10    CONTINUE
C
      P(J1)=ULC
      P(J2)=URC
      DO 20 J=J1+1,J2-1
        P(J)=DHBAR
20    CONTINUE
      WRITE (*,30) P1
30    FORMAT (A80,\)
C
      IF (I2-I1.GT.1) THEN
        DO 40 J=1,80
          P(J)=BLANK
40      CONTINUE
        P(J1)=DVBAR
        P(J2)=DVBAR
        DO 50 I=I1+1,I2-1
          WRITE (*,60) P1
60        FORMAT (A80,\)
50      CONTINUE
        DO 70 J=J1+1,J2-1
          P(J)=DHBAR
70      CONTINUE
      ENDIF
C
      P(J1)=LLC
      P(J2)=LRC
      WRITE (*,60) P1
C
      RETURN
      END
C
C---------------------------------------------------------------------------
      SUBROUTINE WRITE_UNF (A, CD, RD, ZD, JJ, II, LOCAT, LOGBACK)
 
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
        INCLUDE 'SIZES.INC'
        
        LOGICAL*1       LOGBACK
        INTEGER*2       CD, RD, ZD
        INTEGER*4       II,JJ,LOCAT
        REAL*4          A(CD,RD,ZD), TEMP(W_RFX, W_RFY)
        CHARACTER*4     TEXT(4)

        TEXT(1)='COSI'
        TEXT(2)='M  F'
        TEXT(3)='IELD'        
        TEXT(4)='    '
        
        DO 1 I = 1, II
          DO 2 J = 1, JJ
           IF (LOGBACK) A(I,J,1) = 10.**A(I,J,1)
           TEMP(J,I) = A(I,J,1)
2         CONTINUE
1       CONTINUE

        WRITE (LOCAT) IGM1, IGM2, XGM1, XGM2, TEXT, JJ, II, INT4(1)
        WRITE (LOCAT) ((TEMP(I,J), I = 1,JJ), J=1,II)

      RETURN
      END


